from __future__ import annotations

from enum import Enum
from typing import Any, Callable, Type

from django.db import models
from django.db.models import Model
from pydantic import AliasChoices, BaseModel, ConfigDict, Field, model_validator
from typing_extensions import Self


class StringEnum(str, Enum):
    """Python 3.10+ compatible string enum base."""


class AllowedEndpoints(StringEnum):
    STREAM_LIST = "stream_list"
    CREATE = "create"
    DETAIL = "detail"
    UPDATE = "update"
    PATCH = "patch"
    DELETE = "delete"
    LIST = "list"


class IntChoiceSchema(BaseModel):
    value: int
    label: str


class TextChoiceSchema(BaseModel):
    value: str
    label: str


class ChoiceEndpointConfig(BaseModel):
    model_config = ConfigDict(arbitrary_types_allowed=True)

    name: str
    source: type[Any]
    description: str | None = None
    permissions: list[object] = Field(default_factory=list)
    response_schema: Type[BaseModel] | None = None

    @model_validator(mode="after")
    def validate_choice_endpoint(self) -> Self:
        if not self.name.strip():
            raise ValueError("Choice endpoint name cannot be empty.")
        if not hasattr(self.source, "choices"):
            raise ValueError(
                "Choice endpoint source must define a 'choices' attribute."
            )
        if self.response_schema is not None:
            fields = self.response_schema.model_fields
            missing = [field for field in ("value", "label") if field not in fields]
            if missing:
                raise ValueError(
                    "Choice endpoint response_schema must define fields: "
                    + ", ".join(missing)
                )
        return self

    def resolve_response_schema(self) -> Type[BaseModel]:
        if self.response_schema is not None:
            return self.response_schema
        if isinstance(self.source, type) and issubclass(
            self.source, models.IntegerChoices
        ):
            return IntChoiceSchema
        if isinstance(self.source, type) and issubclass(
            self.source, models.TextChoices
        ):
            return TextChoiceSchema
        choices = list(getattr(self.source, "choices"))
        if not choices:
            return TextChoiceSchema
        first_value = choices[0][0]
        if isinstance(first_value, int) and not isinstance(first_value, bool):
            return IntChoiceSchema
        return TextChoiceSchema


class ModelServiceConfig(BaseModel):
    """Configuration for model-based gRPC CRUD service generation."""

    model_config = ConfigDict(arbitrary_types_allowed=True)

    model: Type[Model]
    allowed_endpoints: list[AllowedEndpoints] = Field(
        default=[
            AllowedEndpoints.STREAM_LIST,
            AllowedEndpoints.CREATE,
            AllowedEndpoints.DETAIL,
            AllowedEndpoints.UPDATE,
            AllowedEndpoints.PATCH,
            AllowedEndpoints.DELETE,
            AllowedEndpoints.LIST,
        ],
    )
    list_schema: Type[BaseModel] | None = None
    list_filter: Type[BaseModel] | None = None
    detail_schema: Type[BaseModel] | None = None
    create_schema: Type[BaseModel] | None = None
    update_schema: Type[BaseModel] | None = None
    patch_schema: Type[BaseModel] | None = None
    lookup_field: str = "id"
    queryset: object | Callable[[], object] | None = None
    detail_queryset: object | Callable[[], object] | None = None
    list_pagination_class: object | None = "default"
    list_ordering_class: object | None = None
    list_ordering_fields: list[str] | str = "__all__"
    list_searching_class: object | None = None
    list_search_fields: list[str] = Field(
        default_factory=list,
        validation_alias=AliasChoices("list_search_fields", "list_searching_fields"),
    )
    permissions: list[object] = Field(default_factory=list)
    endpoint_permissions: dict[AllowedEndpoints, list[object]] = Field(
        default_factory=dict
    )
    choice_endpoints: list[ChoiceEndpointConfig] = Field(default_factory=list)

    @model_validator(mode="after")
    def validate_exists_schemas_by_allowed_endpoints(self) -> Self:
        if not self.allowed_endpoints:
            return self
        required_by_endpoint = {
            AllowedEndpoints.STREAM_LIST: ("list_schema",),
            AllowedEndpoints.LIST: ("list_schema",),
            AllowedEndpoints.DETAIL: ("detail_schema",),
            AllowedEndpoints.CREATE: ("create_schema", "detail_schema"),
            AllowedEndpoints.UPDATE: ("update_schema", "detail_schema"),
            AllowedEndpoints.PATCH: ("patch_schema", "detail_schema"),
            AllowedEndpoints.DELETE: (),
        }
        for endpoint in self.allowed_endpoints:
            for field_name in required_by_endpoint[endpoint]:
                if getattr(self, field_name) is None:
                    raise ValueError(
                        f"Field '{field_name}' is required when endpoint '{endpoint.value}' is enabled."
                    )
        return self



from __future__ import annotations

from pathlib import Path
from typing import Iterable

from django.apps import apps
from django.core.management.base import BaseCommand, CommandError


class Command(BaseCommand):
    help = "Initialize gRPC module scaffold inside Django apps."

    def add_arguments(self, parser):
        parser.add_argument(
            "--app",
            action="append",
            default=[],
            help="App label to scaffold (repeatable).",
        )
        parser.add_argument(
            "--all",
            action="store_true",
            help="Scaffold for every installed app.",
        )
        parser.add_argument(
            "--force",
            action="store_true",
            help="Overwrite existing grpc files.",
        )

    def handle(self, *args, **options):
        app_labels = options.get("app", [])
        all_flag = options.get("all", False)
        force = options.get("force", False)

        if not all_flag and not app_labels:
            all_flag = True

        if all_flag and app_labels:
            raise CommandError("Use either --all or --app, not both.")

        if all_flag:
            target_configs = list(apps.get_app_configs())
        else:
            target_configs = self._apps_by_label(app_labels)

        if not target_configs:
            raise CommandError("No matching apps found.")

        created = 0
        skipped = 0
        for app_conf in target_configs:
            result = self._init_grpc_for_app(app_conf, force=force)
            created += result["created"]
            skipped += result["skipped"]

        self.stdout.write(
            f"gRPC scaffold complete. created={created} skipped={skipped}"
        )

    def _apps_by_label(self, labels: Iterable[str]) -> list:
        configs = []
        missing = []
        for label in labels:
            try:
                configs.append(apps.get_app_config(label))
            except LookupError:
                missing.append(label)

        if missing:
            raise CommandError(f"Unknown app label(s): {', '.join(missing)}")
        return configs

    def _init_grpc_for_app(self, app_conf, force: bool) -> dict:
        app_path = Path(app_conf.path)
        grpc_dir = app_path / "grpc"
        proto_dir = grpc_dir / "proto"

        files = {
            grpc_dir / "__init__.py": self._render_grpc_init(app_conf.label),
            grpc_dir / "services.py": self._render_services_stub(app_conf.label),
            proto_dir / "__init__.py": self._render_proto_init(),
        }

        created = 0
        skipped = 0
        for path, content in files.items():
            if path.exists() and not force:
                self.stdout.write(f"skip {path}")
                skipped += 1
                continue

            path.parent.mkdir(parents=True, exist_ok=True)
            path.write_text(content, encoding="utf-8")
            self.stdout.write(f"write {path}")
            created += 1

        return {"created": created, "skipped": skipped}

    def _render_grpc_init(self, label: str) -> str:
        return f'"""gRPC module for `{label}` app."""\n__all__ = []\n'

    def _render_proto_init(self) -> str:
        return '"""gRPC proto package."""\n__all__ = []\n'

    def _render_services_stub(self, label: str) -> str:
        return (
            '"""gRPC service implementations."""\n'
            "\n"
            "# TODO: define your services here with @grpc_service and @grpc_method\n"
        )



from __future__ import annotations

import importlib
import operator
import re
from functools import reduce
from typing import Any, Callable, cast

from django.conf import settings as django_settings
from django.db.models import Q, QuerySet
from django.db.models.constants import LOOKUP_SEP
from pydantic import BaseModel, Field, create_model


def _istartswith(a: str, b: str) -> bool:
    return a.lower().startswith(b.lower())


def _isiexact(a: str, b: str) -> bool:
    return a.lower() == b.lower()


def _isiregex(a: str, b: str) -> bool:
    return bool(re.search(b, a, re.IGNORECASE))


def _isicontains(a: str, b: str) -> bool:
    return b.lower() in a.lower()


class SearchingError(Exception):
    pass


class BaseSearching:
    fields_param_name: str = "search_fields"
    fields_required: bool = True

    @classmethod
    def build_request_schema(
        cls, request_schema: type[BaseModel] | None
    ) -> type[BaseModel]:
        name = (
            f"{request_schema.__name__}WithSearch"
            if request_schema is not None
            else "SearchingRequest"
        )
        if request_schema is None:
            return create_model(name, search=(str | None, Field(default=None)))
        return create_model(
            name,
            __base__=request_schema,
            search=(str | None, Field(default=None)),
        )

    def search(self, items: Any, request: BaseModel) -> Any:
        raise NotImplementedError


class Searching(BaseSearching):
    lookup_prefixes = {
        "^": "istartswith",
        "=": "iexact",
        "@": "search",
        "$": "iregex",
    }
    lookup_prefixes_list = {
        "^": _istartswith,
        "=": _isiexact,
        "$": _isiregex,
    }

    def __init__(self, search_fields: list[str] | None = None) -> None:
        self.search_fields = search_fields or []

    def search(self, items: Any, request: BaseModel) -> Any:
        search_value = cast(str | None, getattr(request, "search", None))
        terms = self._get_search_terms(search_value)
        if not self.search_fields or not terms:
            return items

        if isinstance(items, QuerySet):
            conditions = self._conditions_for_queryset(terms)
            return items.filter(reduce(operator.and_, conditions))

        if isinstance(items, list):
            return self._search_list(items, terms)

        return items

    def _get_search_terms(self, value: str | None) -> list[str]:
        if not value:
            return []
        value = value.replace("\x00", "")
        value = value.replace(",", " ")
        return [term for term in value.split() if term]

    def _construct_search_lookup(self, field_name: str) -> str:
        lookup = self.lookup_prefixes.get(field_name[0])
        if lookup:
            field_name = field_name[1:]
        else:
            lookup = "icontains"
        return LOOKUP_SEP.join([field_name, lookup])

    def _conditions_for_queryset(self, terms: list[str]) -> list[Q]:
        orm_lookups = [
            self._construct_search_lookup(field) for field in self.search_fields
        ]
        conditions: list[Q] = []
        for term in terms:
            queries = [Q(**{lookup: term}) for lookup in orm_lookups]
            conditions.append(reduce(operator.or_, queries))
        return conditions

    def _search_list(self, items: list[Any], terms: list[str]) -> list[Any]:
        lookups = self._construct_list_lookups()
        result: list[Any] = []
        for item in items:
            if self._item_matches(item, lookups, terms):
                result.append(item)
        return result

    def _construct_list_lookups(self) -> dict[str, Callable[[str, str], bool]]:
        def get_lookup(prefix: str) -> Callable[[str, str], bool]:
            return self.lookup_prefixes_list.get(prefix, _isicontains)

        out: dict[str, Callable[[str, str], bool]] = {}
        for field in self.search_fields:
            if self.lookup_prefixes_list.get(field[0]):
                out[field[1:]] = get_lookup(field[0])
            else:
                out[field] = get_lookup(field[0] if field else "")
        return out

    def _item_matches(
        self,
        item: Any,
        lookups: dict[str, Callable[[str, str], bool]],
        terms: list[str],
    ) -> bool:
        is_dict = isinstance(item, dict)
        for term in terms:
            term_match = False
            for field_name, lookup in lookups.items():
                try:
                    raw_value = (
                        item[field_name] if is_dict else getattr(item, field_name)
                    )
                except Exception as exc:
                    raise SearchingError(
                        f"Search field '{field_name}' is not available on list items."
                    ) from exc
                value = str(raw_value)
                if lookup(value, term):
                    term_match = True
                    break
            if not term_match:
                return False
        return True


def resolve_searching_class(value: object | None) -> type[BaseSearching] | None:
    if value is None:
        return None
    if isinstance(value, BaseSearching):
        raise SearchingError(
            "Searching instance is not supported here. "
            "Pass searching class (e.g. Searching) to `list_searching_class` and "
            "pass fields via `list_search_fields`."
        )
    if isinstance(value, str):
        module_path, _, attr = value.rpartition(".")
        if not module_path or not attr:
            raise SearchingError(f"Invalid searching class path: {value}")
        module = importlib.import_module(module_path)
        resolved = getattr(module, attr, None)
        if resolved is None:
            raise SearchingError(f"Searching class '{value}' was not found.")
        value = resolved
    if not isinstance(value, type) or not issubclass(value, BaseSearching):
        raise SearchingError("Searching class must inherit BaseSearching.")
    return value


def get_default_searching_class() -> type[BaseSearching] | None:
    configured = getattr(django_settings, "GRPC_EXTRA", {}) or {}
    value = configured.get("DEFAULT_SEARCHING_CLASS", "grpc_extra.searching.Searching")
    return resolve_searching_class(value)



import dataclasses
from typing import Type

from pydantic import BaseModel


@dataclasses.dataclass
class MethodParameter:
    method_name: str
    request_schema: Type[BaseModel] | None = None
    response_schema: Type[BaseModel] | None = None
    description: str | None = None

    def dict(self) -> dict:
        return dataclasses.asdict(self)


__all__ = ["MethodParameter"]



from __future__ import annotations

from collections.abc import Iterable
from typing import Any, ClassVar, Literal, cast

from django.db.models import Q
from pydantic import BaseModel

LookupOperator = Literal["exact", "in", "not_in", "lt", "gt", "lte", "gte"]
ExpressionConnector = Literal["AND", "OR", "XOR"]


class ModelFilterSchema(BaseModel):
    """Base class for declarative queryset filters used by ModelService list methods."""

    ignore_none: ClassVar[bool] = True
    expression_connector: ClassVar[ExpressionConnector] = "AND"

    def custom_expression(self) -> Q:
        raise NotImplementedError

    def to_q(self) -> Q:
        try:
            return self.custom_expression()
        except NotImplementedError:
            return self._connect_fields()

    def filter_queryset(self, queryset):
        return queryset.filter(self.to_q())

    def _resolve_field_q(self, field_name: str, field_value) -> Q:
        custom_resolver = getattr(self, f"filter_{field_name}", None)
        if callable(custom_resolver):
            return cast(Q, custom_resolver(field_value))

        field = self.__class__.model_fields[field_name]
        extra_raw = field.json_schema_extra
        extra: dict[str, Any] = extra_raw if isinstance(extra_raw, dict) else {}
        target_field = cast(str, extra.get("field", field_name))
        lookup = cast(str | None, extra.get("lookup"))
        op = cast(LookupOperator, extra.get("op", "exact"))
        exclude = bool(extra.get("exclude", False))

        if lookup is not None:
            lookup_name = (
                f"{target_field}{lookup}" if lookup.startswith("__") else lookup
            )
            q = Q(**{lookup_name: field_value})
            return ~q if exclude else q

        if op == "exact":
            q = Q(**{target_field: field_value})
        elif op == "in":
            values = list(cast(Iterable, field_value))
            q = Q(**{f"{target_field}__in": values})
        elif op == "not_in":
            values = list(cast(Iterable, field_value))
            q = ~Q(**{f"{target_field}__in": values})
        elif op == "lt":
            q = Q(**{f"{target_field}__lt": field_value})
        elif op == "gt":
            q = Q(**{f"{target_field}__gt": field_value})
        elif op == "lte":
            q = Q(**{f"{target_field}__lte": field_value})
        elif op == "gte":
            q = Q(**{f"{target_field}__gte": field_value})
        else:
            raise ValueError(
                f"Unsupported filter operator '{op}' for field '{field_name}'."
            )

        return ~q if exclude else q

    def _connect_fields(self) -> Q:
        q = Q()
        connector = self.expression_connector
        for field_name in self.__class__.model_fields:
            filter_value = getattr(self, field_name)
            if filter_value is None and self.ignore_none:
                continue
            field_q = self._resolve_field_q(field_name, filter_value)
            if connector == "OR":
                q = q | field_q
            elif connector == "XOR":
                q = q ^ field_q
            else:
                q = q & field_q
        return q



from __future__ import annotations

import importlib
import inspect
from abc import ABC, abstractmethod
from typing import Any, Callable, cast

import grpc


class AuthError(Exception):
    pass


class GrpcAuthBase(ABC):
    """Base callable auth backend contract for gRPC metadata/context."""

    def __call__(self, context, method: str, request=None) -> Any:
        return self.authenticate(context, method, request)

    @abstractmethod
    def authenticate(self, context, method: str, request=None) -> Any:
        raise NotImplementedError


class GrpcBearerAuthBase(GrpcAuthBase, ABC):
    """Bearer token auth helper for gRPC metadata-based backends."""

    scheme: str = "Bearer"
    header: str = "Authorization"

    def __call__(self, context, method: str, request=None) -> Any:
        token = self._extract_token(context)
        if not token:
            return None
        return self.authenticate_token(context, token, method, request)

    def authenticate_token(self, context, token: str, method: str, request=None) -> Any:
        # Backward-compatible fallback: subclasses may override `authenticate`
        # with `(context, token, method, request)` signature.
        return cast(Any, self.authenticate)(context, token, method, request)

    def _extract_token(self, context) -> str | None:
        raw_value = self._metadata_value(context, self.header)
        if not raw_value:
            return None
        if not self.scheme:
            token = raw_value.strip()
            if not token:
                return None
            return token
        parts = raw_value.split(" ")
        if not parts:
            return None
        if parts[0].lower() != self.scheme.lower():
            return None
        token = " ".join(parts[1:]).strip()
        if not token:
            return None
        return token

    def _metadata_value(self, context, header: str) -> str | None:
        invocation_metadata = getattr(context, "invocation_metadata", None)
        if not callable(invocation_metadata):
            return None
        for key, value in invocation_metadata() or ():
            if str(key).lower() == header.lower():
                return str(value)
        return None


def resolve_auth_backend(backend: str | Callable | None) -> Callable | None:
    if backend is None:
        return None
    if callable(backend):
        return _normalize_auth_backend_callable(backend)
    module_path, _, attr = backend.rpartition(".")
    if not module_path or not attr:
        raise AuthError(f"Invalid auth backend path: {backend}")
    module = importlib.import_module(module_path)
    resolved = getattr(module, attr, None)
    if not callable(resolved):
        raise AuthError(f"Auth backend '{backend}' is not callable")
    return _normalize_auth_backend_callable(cast(Callable[..., Any], resolved))


def _normalize_auth_backend_callable(backend: Callable[..., Any]) -> Callable[..., Any]:
    if inspect.isclass(backend):
        try:
            instance = backend()
        except TypeError as exc:
            raise AuthError(
                "Auth backend class must be instantiable without constructor arguments."
            ) from exc
        if not callable(instance):
            raise AuthError("Auth backend instance is not callable")
        return cast(Callable[..., Any], instance)
    return backend


class AuthInterceptor(grpc.ServerInterceptor):
    def __init__(self, backend: Callable):
        self.backend = backend

    def intercept_service(self, continuation, handler_call_details):
        handler = continuation(handler_call_details)
        if handler is None:
            return None

        method = handler_call_details.method

        def _check(context, request=None):
            result = self.backend(context, method, request)
            if result is False or result is None:
                context.abort(grpc.StatusCode.UNAUTHENTICATED, "Unauthorized")

        if handler.unary_unary:

            def unary_unary(request, context):
                _check(context, request)
                return handler.unary_unary(request, context)

            return grpc.unary_unary_rpc_method_handler(
                unary_unary,
                request_deserializer=handler.request_deserializer,
                response_serializer=handler.response_serializer,
            )

        if handler.unary_stream:

            def unary_stream(request, context):
                _check(context, request)
                return handler.unary_stream(request, context)

            return grpc.unary_stream_rpc_method_handler(
                unary_stream,
                request_deserializer=handler.request_deserializer,
                response_serializer=handler.response_serializer,
            )

        if handler.stream_unary:

            def stream_unary(request_iterator, context):
                _check(context)
                return handler.stream_unary(request_iterator, context)

            return grpc.stream_unary_rpc_method_handler(
                stream_unary,
                request_deserializer=handler.request_deserializer,
                response_serializer=handler.response_serializer,
            )

        if handler.stream_stream:

            def stream_stream(request_iterator, context):
                _check(context)
                return handler.stream_stream(request_iterator, context)

            return grpc.stream_stream_rpc_method_handler(
                stream_stream,
                request_deserializer=handler.request_deserializer,
                response_serializer=handler.response_serializer,
            )

        return handler



from __future__ import annotations

from pathlib import Path
from typing import Iterable

from django.apps import AppConfig, apps
from django.conf import settings as django_settings
from django.core.management import call_command
from django.core.management.base import BaseCommand, CommandError
from django.utils.module_loading import import_string

from grpc_extra.main import GrpcExtra
from grpc_extra.registry import ServiceDefinition, registry
from grpc_extra.sdk.generators import BaseClientSDKGenerator, SDKGenerationError
from grpc_extra.utils import normalize_proto_path


DEFAULT_SDK_GENERATORS = {
    "python": "grpc_extra.sdk.generators.PythonClientSDKGenerator",
    "php": "grpc_extra.sdk.generators.PhpClientSDKGenerator",
}


class Command(BaseCommand):
    help = "Generate language-specific client SDK from project proto files."

    def add_arguments(self, parser):
        parser.add_argument("--language", required=True, help="Target SDK language.")
        parser.add_argument(
            "--out",
            default="generated-sdks",
            help="Directory where SDK artifact will be generated.",
        )
        parser.add_argument(
            "--name",
            default=None,
            help="SDK package/directory name. Defaults to django-grpc-extra-<language>-client-sdk.",
        )
        parser.add_argument(
            "--app",
            action="append",
            default=[],
            help="App label to include (repeatable).",
        )
        parser.add_argument(
            "--all",
            action="store_true",
            help="Include all installed apps.",
        )
        parser.add_argument(
            "--skip-proto",
            action="store_true",
            help="Skip proto regeneration and use existing proto files.",
        )

    def handle(self, *args, **options):
        language = str(options["language"]).lower()
        out_dir = Path(options["out"])
        name = options.get("name") or f"django-grpc-extra-{language}-client-sdk"
        app_labels = options.get("app", [])
        include_all = options.get("all", False)
        skip_proto = options.get("skip_proto", False)

        if include_all and app_labels:
            raise CommandError("Use either --all or --app, not both.")

        if not include_all and not app_labels:
            include_all = True

        if not skip_proto:
            call_command(
                "generate_proto",
                all=include_all,
                app=app_labels,
                no_compile=True,
            )

        app_configs = (
            list(apps.get_app_configs())
            if include_all
            else [apps.get_app_config(label) for label in app_labels]
        )
        app_by_label = {app.label: app for app in app_configs}

        registry.clear()
        GrpcExtra().auto_discover_services()
        definitions = list(registry.all())
        if not definitions:
            raise CommandError("No decorated services found.")

        proto_files = self._collect_proto_files(definitions, app_by_label)
        if not proto_files:
            raise CommandError("No proto files found for selected apps.")

        include_root = self._include_root(
            self._service_app_configs(definitions, app_by_label)
        )
        generator = self._resolve_generator(language)
        try:
            target = generator.generate(
                proto_files=proto_files,
                out_dir=out_dir,
                sdk_name=name,
                include_root=include_root,
            )
        except SDKGenerationError as exc:
            raise CommandError(str(exc)) from exc

        self.stdout.write(f"client sdk generated: {target}")

    def _collect_proto_files(
        self,
        definitions: Iterable[ServiceDefinition],
        app_by_label: dict[str, AppConfig],
    ) -> list[Path]:
        proto_files: list[Path] = []
        seen: set[Path] = set()
        for definition in definitions:
            app_conf = app_by_label.get(definition.meta.app_label)
            if app_conf is None:
                continue
            if not definition.meta.proto_path:
                continue
            proto_rel = normalize_proto_path(definition.meta.proto_path)
            proto_abs = Path(app_conf.path) / proto_rel
            if proto_abs.exists() and proto_abs not in seen:
                proto_files.append(proto_abs)
                seen.add(proto_abs)
        return proto_files

    def _include_root(self, app_configs: list[AppConfig]) -> Path:
        roots = {Path(app.path).parent for app in app_configs}
        if len(roots) != 1:
            raise CommandError(
                "Apps have different include roots; use --app for a single app root."
            )
        return roots.pop()

    def _service_app_configs(
        self,
        definitions: Iterable[ServiceDefinition],
        app_by_label: dict[str, AppConfig],
    ) -> list[AppConfig]:
        selected: list[AppConfig] = []
        seen: set[str] = set()
        for definition in definitions:
            if not definition.meta.proto_path:
                continue
            label = definition.meta.app_label
            if label in seen:
                continue
            app_conf = app_by_label.get(label)
            if app_conf is None:
                continue
            selected.append(app_conf)
            seen.add(label)
        return selected

    def _resolve_generator(self, language: str) -> BaseClientSDKGenerator:
        configured = getattr(django_settings, "GRPC_EXTRA", {}) or {}
        mapping = dict(DEFAULT_SDK_GENERATORS)
        mapping.update(configured.get("SDK_GENERATORS", {}))
        path = mapping.get(language)
        if not path:
            raise CommandError(
                f"SDK generator is not configured for language '{language}'."
            )
        value = import_string(path)
        instance = value() if isinstance(value, type) else value
        if not isinstance(instance, BaseClientSDKGenerator):
            raise CommandError(
                f"SDK generator '{path}' must inherit BaseClientSDKGenerator."
            )
        return instance



from __future__ import annotations

from dataclasses import dataclass
from datetime import date, datetime, time
from decimal import Decimal
from pathlib import Path
from types import UnionType
from typing import Any, Iterable, Union, cast, get_args, get_origin
from uuid import UUID

from django.apps import apps
from django.conf import settings as django_settings
from django.core.management.base import BaseCommand, CommandError
from pydantic import BaseModel

from grpc_extra.main import GrpcExtra
from grpc_extra.registry import MethodMeta, ServiceDefinition, registry
from grpc_extra.utils import normalize_proto_path

try:
    from enum import Enum
except ImportError:  # pragma: no cover - stdlib always provides Enum
    Enum = object  # type: ignore[misc,assignment]


@dataclass
class FieldSpec:
    name: str
    number: int
    type_name: str
    repeated: bool = False
    optional: bool = False
    description: str | None = None


class ProtoTypeError(CommandError):
    pass


class ProtoBuilder:
    def __init__(self, package: str) -> None:
        self.package = package
        self.imports: set[str] = set()
        self.messages: dict[str, list[FieldSpec]] = {}
        self.message_descriptions: dict[str, str] = {}
        self._message_models: dict[str, type[BaseModel]] = {}
        self.enums: dict[str, list[str]] = {}

    def register_message(
        self, model: type[BaseModel], *, name: str | None = None
    ) -> str:
        message_name = name or model.__name__
        existing_model = self._message_models.get(message_name)
        if existing_model is not None and existing_model is not model:
            if not self._is_same_message_shape(existing_model, model):
                raise ProtoTypeError(
                    f"Message name conflict for '{message_name}': "
                    f"'{existing_model.__name__}' vs '{model.__name__}'."
                )
            return message_name
        self._message_models[message_name] = model
        model_doc = getattr(model, "__doc__", None)
        if model_doc:
            self.message_descriptions[message_name] = str(model_doc).strip()
        if message_name in self.messages:
            return message_name
        fields: list[FieldSpec] = []
        field_number = 1
        for field_name, field in model.model_fields.items():
            type_name, repeated, optional = self._proto_type(field.annotation)
            field_alias = field.alias or field_name
            fields.append(
                FieldSpec(
                    name=field_alias,
                    number=field_number,
                    type_name=type_name,
                    repeated=repeated,
                    optional=optional,
                    description=field.description or field.title,
                )
            )
            field_number += 1
        self.messages[message_name] = fields
        return message_name

    def _is_same_message_shape(
        self, left_model: type[BaseModel], right_model: type[BaseModel]
    ) -> bool:
        return self._message_shape_signature(
            left_model
        ) == self._message_shape_signature(right_model)

    def _message_shape_signature(
        self, model: type[BaseModel]
    ) -> tuple[tuple[str, Any, str], ...]:
        signature: list[tuple[str, Any, str]] = []
        for field_name, field in model.model_fields.items():
            signature.append((field_name, field.annotation, field.alias or field_name))
        return tuple(signature)

    def register_enum(self, enum_cls: type[Enum]) -> None:
        if enum_cls.__name__ in self.enums:
            return
        values = [member.name for member in enum_cls]
        self.enums[enum_cls.__name__] = values

    def _proto_type(self, annotation) -> tuple[str, bool, bool]:
        origin = get_origin(annotation)
        args = get_args(annotation)

        if origin is list:
            if not args:
                raise ProtoTypeError("List[T] must specify item type.")
            item_type, _, _ = self._proto_type(args[0])
            return item_type, True, False

        if origin is dict:
            self.imports.add("google/protobuf/struct.proto")
            return "google.protobuf.Struct", False, False

        if origin in (Union, UnionType):
            non_none = [arg for arg in args if arg is not type(None)]
            if len(non_none) == 1:
                type_name, repeated, _ = self._proto_type(non_none[0])
                return type_name, repeated, not repeated
            raise ProtoTypeError("Union types are not supported in proto generation.")

        if isinstance(annotation, type):
            if issubclass(annotation, BaseModel):
                message_name = self.register_message(annotation)
                return message_name, False, False
            if issubclass(annotation, Enum):
                self.register_enum(annotation)
                return annotation.__name__, False, False
            if annotation is dict:
                self.imports.add("google/protobuf/struct.proto")
                return "google.protobuf.Struct", False, False
            if annotation is str:
                return "string", False, False
            if annotation is int:
                return "int64", False, False
            if annotation is float:
                return "double", False, False
            if annotation is bool:
                return "bool", False, False
            if annotation is bytes:
                return "bytes", False, False
            if annotation is datetime:
                self.imports.add("google/protobuf/timestamp.proto")
                return "google.protobuf.Timestamp", False, False
            if annotation is date:
                self.imports.add("google/type/date.proto")
                return "google.type.Date", False, False
            if annotation is time:
                self.imports.add("google/type/timeofday.proto")
                return "google.type.TimeOfDay", False, False
            if annotation is Decimal:
                return "string", False, False
            if annotation is UUID:
                return "string", False, False
            raise ProtoTypeError(
                f"Unsupported type '{annotation.__name__}' for proto generation."
            )

        raise ProtoTypeError(f"Unsupported type annotation '{annotation}'.")


class Command(BaseCommand):
    help = "Generate .proto files from @grpc_service and @grpc_method metadata."

    def add_arguments(self, parser):
        parser.add_argument(
            "--app",
            action="append",
            default=[],
            help="App label to generate (repeatable).",
        )
        parser.add_argument(
            "--all",
            action="store_true",
            help="Generate for every installed app.",
        )
        parser.add_argument(
            "--force",
            action="store_true",
            help="Overwrite existing proto files.",
        )
        parser.add_argument(
            "--no-compile",
            action="store_true",
            help="Skip pb2/pb2_grpc generation via grpc_tools.",
        )
        parser.add_argument(
            "--pyi",
            action="store_true",
            help="Generate .pyi stubs via grpc_tools (requires --no-compile=false).",
        )

    def handle(self, *args, **options):
        app_labels = options.get("app", [])
        all_flag = options.get("all", False)
        force = options.get("force", False)
        no_compile = options.get("no_compile", False)
        pyi = options.get("pyi", False)

        if not all_flag and not app_labels:
            all_flag = True

        if all_flag and app_labels:
            raise CommandError("Use either --all or --app, not both.")

        registry.clear()
        GrpcExtra().auto_discover_services()

        if all_flag:
            target_configs = list(apps.get_app_configs())
        else:
            target_configs = self._apps_by_label(app_labels)

        definitions = list(registry.all())
        if not definitions:
            raise CommandError("No decorated services found.")

        by_app: dict[str, list[ServiceDefinition]] = {}
        for definition in definitions:
            by_app.setdefault(definition.meta.app_label, []).append(definition)

        created = 0
        skipped = 0
        compiled = 0
        for app_conf in target_configs:
            app_defs = by_app.get(app_conf.label, [])
            proto_files: list[Path] = []
            for path, content in self._render_protos(app_conf.path, app_defs):
                if path.exists() and not force:
                    self.stdout.write(f"skip {path}")
                    skipped += 1
                    proto_files.append(path)
                    continue
                path.parent.mkdir(parents=True, exist_ok=True)
                path.write_text(content, encoding="utf-8")
                self.stdout.write(f"write {path}")
                created += 1
                proto_files.append(path)

            if proto_files and not no_compile:
                compiled += self._compile_protos(app_conf.path, proto_files, pyi=pyi)

        self.stdout.write(
            "proto generation complete. "
            f"created={created} skipped={skipped} compiled={compiled}"
        )

    def _grpc_extra_settings(self) -> dict:
        return getattr(django_settings, "GRPC_EXTRA", {}) or {}

    def _schema_suffix_strip(self) -> tuple[str, ...]:
        configured = self._grpc_extra_settings().get("SCHEMA_SUFFIX_STRIP", ("Schema",))
        if isinstance(configured, str):
            return (configured,)
        return tuple(configured)

    def _request_suffix(self) -> str:
        return cast(str, self._grpc_extra_settings().get("REQUEST_SUFFIX", "Request"))

    def _response_suffix(self) -> str:
        return cast(str, self._grpc_extra_settings().get("RESPONSE_SUFFIX", "Response"))

    def _apps_by_label(self, labels: Iterable[str]) -> list:
        configs = []
        missing = []
        for label in labels:
            try:
                configs.append(apps.get_app_config(label))
            except LookupError:
                missing.append(label)

        if missing:
            raise CommandError(f"Unknown app label(s): {', '.join(missing)}")
        return configs

    def _render_protos(
        self, app_path: str, definitions: list[ServiceDefinition]
    ) -> Iterable[tuple[Path, str]]:
        by_path: dict[str, list[ServiceDefinition]] = {}
        for definition in definitions:
            if not definition.meta.proto_path:
                raise CommandError(
                    f"Service '{definition.meta.name}' must define proto_path."
                )
            proto_path = normalize_proto_path(definition.meta.proto_path)
            by_path.setdefault(proto_path, []).append(definition)

        for proto_path, service_defs in by_path.items():
            if not proto_path:
                continue
            content = self._build_proto(service_defs)
            yield Path(app_path) / proto_path, content

    def _build_proto(self, service_defs: list[ServiceDefinition]) -> str:
        package = self._package_from(service_defs)
        builder = ProtoBuilder(package=package)
        service_blocks = [self._render_service(defn, builder) for defn in service_defs]
        imports = self._render_imports(builder.imports)
        enums = self._render_enums(builder.enums)
        messages = self._render_messages(
            builder.messages,
            builder.message_descriptions,
        )

        parts = [
            'syntax = "proto3";',
            f"package {package};",
            "",
            imports,
            enums,
            messages,
            "\n\n".join(service_blocks),
            "",
        ]
        return "\n".join(part for part in parts if part)

    def _package_from(self, service_defs: list[ServiceDefinition]) -> str:
        packages = {definition.meta.package for definition in service_defs}
        packages.discard(None)
        if not packages:
            return "grpc"
        if len(packages) > 1:
            raise CommandError("Multiple packages in the same proto file.")
        return cast(str, packages.pop())

    def _render_imports(self, imports: set[str]) -> str:
        if not imports:
            return ""
        return "\n".join(f'import "{path}";' for path in sorted(imports)) + "\n"

    def _render_enums(self, enums: dict[str, list[str]]) -> str:
        if not enums:
            return ""
        blocks = []
        for name, values in enums.items():
            lines = [f"enum {name} {{"] + [
                f"  {value} = {index};" for index, value in enumerate(values)
            ]
            lines.append("}")
            blocks.append("\n".join(lines))
        return "\n\n".join(blocks) + "\n"

    def _render_messages(
        self,
        messages: dict[str, list[FieldSpec]],
        message_descriptions: dict[str, str] | None = None,
    ) -> str:
        if not messages:
            return ""
        message_descriptions = message_descriptions or {}
        blocks = []
        for name, fields in messages.items():
            lines = [f"message {name} {{"]
            message_comment = self._render_comment(
                message_descriptions.get(name),
            )
            if message_comment:
                lines = [message_comment, *lines]
            for field in fields:
                repeated = "repeated " if field.repeated else ""
                optional = "optional " if field.optional and not field.repeated else ""
                field_comment = self._render_comment(
                    getattr(field, "description", None), indent="  "
                )
                if field_comment:
                    lines.append(field_comment)
                lines.append(
                    f"  {optional}{repeated}{field.type_name} {field.name} = {field.number};"
                )
            lines.append("}")
            blocks.append("\n".join(lines))
        return "\n\n".join(blocks) + "\n"

    def _render_service(
        self, definition: ServiceDefinition, builder: ProtoBuilder
    ) -> str:
        lines = [f"service {definition.meta.name} {{"]
        service_comment = self._render_comment(definition.meta.description)
        if service_comment:
            lines = [service_comment, *lines]
        methods = definition.methods or self._fallback_methods(definition.service)
        for method in methods:
            request = self._message_name(
                method.request_schema,
                builder,
                kind="request",
                service_name=definition.meta.name,
                method_name=method.name,
            )
            response = self._message_name(
                method.response_schema,
                builder,
                kind="response",
                service_name=definition.meta.name,
                method_name=method.name,
            )
            req_stream = "stream " if method.client_streaming else ""
            res_stream = "stream " if method.server_streaming else ""
            method_comment = self._render_comment(method.description, indent="  ")
            if method_comment:
                lines.append(method_comment)
            lines.append(
                f"  rpc {method.name} ({req_stream}{request}) returns ({res_stream}{response});"
            )
        lines.append("}")
        return "\n".join(lines)

    def _render_comment(self, text: str | None, *, indent: str = "") -> str:
        if not text:
            return ""
        lines = [
            line.strip() for line in str(text).strip().splitlines() if line.strip()
        ]
        if not lines:
            return ""
        return "\n".join(f"{indent}// {line}" for line in lines)

    def _message_name(
        self,
        schema: type[BaseModel] | None,
        builder: ProtoBuilder,
        *,
        kind: str,
        service_name: str,
        method_name: str,
    ) -> str:
        if schema is None:
            builder.imports.add("google/protobuf/empty.proto")
            return "google.protobuf.Empty"
        if kind not in {"request", "response"}:
            raise ProtoTypeError(f"Unknown schema kind '{kind}'.")
        suffix = (
            self._request_suffix() if kind == "request" else self._response_suffix()
        )
        base = f"{service_name}{method_name}"
        message_name = f"{base}{suffix}" if suffix else base
        return builder.register_message(schema, name=message_name)

    def _schema_base_name(self, schema_name: str) -> str:
        for suffix in self._schema_suffix_strip():
            if suffix and schema_name.endswith(suffix):
                stripped = schema_name[: -len(suffix)]
                if stripped:
                    return stripped
        return schema_name

    def _fallback_methods(self, service: type) -> list[MethodMeta]:
        return []

    def _compile_protos(
        self, app_path: str, proto_files: Iterable[Path], *, pyi: bool
    ) -> int:
        try:
            import grpc_tools
            from grpc_tools import protoc
        except ImportError as exc:
            raise CommandError(
                "grpc_tools is required for pb2 generation. "
                "Install grpcio-tools or use --no-compile."
            ) from exc

        app_root = Path(app_path).parent
        grpc_tools_include = self._grpc_tools_include_path(grpc_tools)
        compiled = 0
        for proto_file in proto_files:
            args = [
                "protoc",
                f"-I{app_root}",
            ]
            if grpc_tools_include is not None:
                args.append(f"-I{grpc_tools_include}")
            args.extend(
                [
                    f"--python_out={app_root}",
                    f"--grpc_python_out={app_root}",
                ]
            )
            if pyi:
                args.append(f"--pyi_out={app_root}")
            args.append(str(proto_file))
            result = protoc.main(args)
            if result != 0:
                raise CommandError(f"protoc failed for {proto_file}")
            compiled += 1
        return compiled

    def _grpc_tools_include_path(self, grpc_tools_module: object) -> Path | None:
        module_file = getattr(grpc_tools_module, "__file__", None)
        if not module_file:
            return None
        include_path = Path(module_file).resolve().parent / "_proto"
        if include_path.exists():
            return include_path
        return None



from __future__ import annotations

from collections.abc import Callable, Iterable, Iterator, Mapping
from typing import Any, NoReturn

from .codec import (
    decode_request_iter,
    decode_request_value,
    encode_response_value,
)
from .exceptions import MappedError, resolve_exception_mapper
from .permissions import BasePermission
from .registry import MethodMeta, ServiceDefinition


class ServiceRuntimeAdapter:
    """Adapts business methods to grpc-compatible handlers."""

    def __init__(
        self,
        definition: ServiceDefinition,
        pb2_module,
        *,
        exception_mapper: Callable[[Exception], MappedError] | str | None = None,
    ) -> None:
        self.definition = definition
        self.pb2_module = pb2_module
        self.exception_mapper = resolve_exception_mapper(exception_mapper)

    def apply(self, servicer: object) -> None:
        service_desc = self.pb2_module.DESCRIPTOR.services_by_name.get(
            self.definition.meta.name
        )
        if service_desc is None:
            return

        for method_meta in self.definition.methods:
            method_desc = service_desc.methods_by_name.get(method_meta.name)
            if method_desc is None:
                continue
            response_cls = getattr(self.pb2_module, method_desc.output_type.name, None)
            if response_cls is None:
                continue
            wrapper = self._build_wrapper(servicer, method_meta, response_cls)
            if wrapper is None:
                continue
            setattr(servicer, method_meta.name, wrapper)

    def _build_wrapper(
        self,
        servicer: object,
        method_meta: MethodMeta,
        response_pb2_cls: type,
    ) -> Callable | None:
        method = getattr(servicer, method_meta.handler_name, None)
        if method is None:
            return None
        if method_meta.client_streaming and method_meta.server_streaming:
            return self._wrap_stream_stream(
                method, method_meta, response_pb2_cls, servicer
            )
        if method_meta.client_streaming:
            return self._wrap_stream_unary(
                method, method_meta, response_pb2_cls, servicer
            )
        if method_meta.server_streaming:
            return self._wrap_unary_stream(
                method, method_meta, response_pb2_cls, servicer
            )
        return self._wrap_unary_unary(method, method_meta, response_pb2_cls, servicer)

    def _wrap_unary_unary(
        self,
        method: Callable,
        method_meta: MethodMeta,
        response_pb2_cls: type,
        service: object | None = None,
    ) -> Callable:
        def wrapper(request, context):
            try:
                decoded = decode_request_value(request, method_meta.request_schema)
                self._check_service_permissions(decoded, context, method_meta, service)
                self._check_method_permissions(decoded, context, method_meta, service)
                result = method(decoded, context)
                self._check_obj_permissions(
                    decoded, context, method_meta, service, result
                )
                result = self._apply_searching_ordering(result, decoded, method_meta)
                if method_meta.pagination_class is not None:
                    result = method_meta.pagination_class.paginate(result, decoded)
                return encode_response_value(
                    result, method_meta.response_schema, response_pb2_cls
                )
            except Exception as exc:
                self._abort(context, exc)

        return wrapper

    def _wrap_unary_stream(
        self,
        method: Callable,
        method_meta: MethodMeta,
        response_pb2_cls: type,
        service: object | None = None,
    ) -> Callable:
        def wrapper(request, context) -> Iterator[Any]:
            try:
                decoded = decode_request_value(request, method_meta.request_schema)
                self._check_service_permissions(decoded, context, method_meta, service)
                self._check_method_permissions(decoded, context, method_meta, service)
                result = method(decoded, context)
                result = self._apply_searching_ordering(result, decoded, method_meta)
                if not isinstance(result, Iterable):
                    raise TypeError(
                        f"Method '{method_meta.name}' must return iterable for server streaming."
                    )
                return self._encode_stream(
                    result,
                    method_meta,
                    response_pb2_cls,
                    context,
                    decoded,
                    service,
                )
            except Exception as exc:
                self._abort(context, exc)

        return wrapper

    def _wrap_stream_unary(
        self,
        method: Callable,
        method_meta: MethodMeta,
        response_pb2_cls: type,
        service: object | None = None,
    ) -> Callable:
        def wrapper(request_iterator, context):
            try:
                self._check_service_permissions(None, context, method_meta, service)
                self._check_method_permissions(None, context, method_meta, service)
                decoded_iter = decode_request_iter(
                    request_iterator, method_meta.request_schema
                )
                result = method(decoded_iter, context)
                self._check_obj_permissions(None, context, method_meta, service, result)
                if method_meta.pagination_class is not None:
                    raise TypeError(
                        "Pagination is not supported for stream-unary methods."
                    )
                return encode_response_value(
                    result, method_meta.response_schema, response_pb2_cls
                )
            except Exception as exc:
                self._abort(context, exc)

        return wrapper

    def _wrap_stream_stream(
        self,
        method: Callable,
        method_meta: MethodMeta,
        response_pb2_cls: type,
        service: object | None = None,
    ) -> Callable:
        def wrapper(request_iterator, context) -> Iterator[Any]:
            try:
                self._check_service_permissions(None, context, method_meta, service)
                self._check_method_permissions(None, context, method_meta, service)
                decoded_iter = decode_request_iter(
                    request_iterator, method_meta.request_schema
                )
                result = method(decoded_iter, context)
                if not isinstance(result, Iterable):
                    raise TypeError(
                        f"Method '{method_meta.name}' must return iterable for bidirectional streaming."
                    )
                return self._encode_stream(
                    result,
                    method_meta,
                    response_pb2_cls,
                    context,
                    None,
                    service,
                )
            except Exception as exc:
                self._abort(context, exc)

        return wrapper

    def _encode_stream(
        self,
        result: Iterable[Any],
        method_meta: MethodMeta,
        response_pb2_cls: type,
        context,
        request=None,
        service: object | None = None,
    ) -> Iterator[Any]:
        try:
            for raw_item in result:
                self._check_obj_permissions(
                    request, context, method_meta, service, raw_item
                )
                item = encode_response_value(
                    raw_item, method_meta.response_schema, response_pb2_cls
                )
                yield item
        except Exception as exc:
            self._abort(context, exc)

    def _check_service_permissions(
        self,
        request,
        context,
        method_meta: MethodMeta,
        service: object | None,
    ) -> None:
        if method_meta.permissions_overridden:
            return
        permissions = self.definition.meta.permissions
        if not permissions:
            return
        target = service or self.definition.service
        self._run_has_perm(permissions, request, context, target, method_meta)

    def _check_method_permissions(
        self,
        request,
        context,
        method_meta: MethodMeta,
        service: object | None,
    ) -> None:
        permissions = method_meta.permissions
        if not permissions:
            return
        target = service or self.definition.service
        self._run_has_perm(permissions, request, context, target, method_meta)

    def _check_obj_permissions(
        self,
        request,
        context,
        method_meta: MethodMeta,
        service: object | None,
        obj: Any,
    ) -> None:
        target = service or self.definition.service
        method_permissions = method_meta.permissions
        if method_permissions:
            self._run_has_obj_perm(
                method_permissions, request, context, target, method_meta, obj
            )
        if (
            self._is_detail_method(method_meta)
            and not method_meta.permissions_overridden
        ):
            service_permissions = self.definition.meta.permissions
            if service_permissions:
                self._run_has_obj_perm(
                    service_permissions, request, context, target, method_meta, obj
                )

    def _run_has_perm(
        self,
        permissions: tuple[BasePermission, ...],
        request,
        context,
        service: object,
        method_meta: MethodMeta,
    ) -> None:
        for permission in permissions:
            if not permission.has_perm(request, context, service, method_meta):
                raise PermissionError(
                    getattr(permission, "message", "Permission denied.")
                )

    def _run_has_obj_perm(
        self,
        permissions: tuple[BasePermission, ...],
        request,
        context,
        service: object,
        method_meta: MethodMeta,
        obj: Any,
    ) -> None:
        for permission in permissions:
            if not permission.has_obj_perm(request, context, service, method_meta, obj):
                raise PermissionError(
                    getattr(permission, "message", "Permission denied.")
                )

    def _apply_searching_ordering(self, result, decoded, method_meta: MethodMeta):
        searching_handler = method_meta.searching_handler
        if searching_handler is not None:
            result = self._apply_modifier_to_items(
                result,
                lambda items: searching_handler.search(items, decoded),
            )
        ordering_handler = method_meta.ordering_handler
        if ordering_handler is not None:
            result = self._apply_modifier_to_items(
                result,
                lambda items: ordering_handler.order(items, decoded),
            )
        return result

    def _apply_modifier_to_items(self, result, modifier: Callable[[Any], Any]):
        if isinstance(result, Mapping) and "items" in result:
            payload = dict(result)
            payload["items"] = modifier(payload["items"])
            return payload
        return modifier(result)

    def _is_detail_method(self, method_meta: MethodMeta) -> bool:
        method_name = method_meta.name.replace("-", "_").upper()
        if method_name in {"DETAIL", "GET"}:
            return True
        handler_name = method_meta.handler_name.replace("-", "_").upper()
        return handler_name in {"DETAIL", "GET"}

    def _abort(self, context, exc: Exception) -> NoReturn:
        mapped = self.exception_mapper(exc)
        context.abort(mapped.code, mapped.message)
        raise RuntimeError("gRPC context.abort returned unexpectedly")



from __future__ import annotations

import importlib
from collections.abc import Iterable
from operator import attrgetter, itemgetter
from typing import Any, cast

from django.conf import settings as django_settings
from django.db.models import QuerySet
from pydantic import BaseModel, Field, create_model


class OrderingError(Exception):
    pass


class BaseOrdering:
    fields_param_name: str = "ordering_fields"
    fields_required: bool = True

    @classmethod
    def build_request_schema(
        cls, request_schema: type[BaseModel] | None
    ) -> type[BaseModel]:
        name = (
            f"{request_schema.__name__}WithOrdering"
            if request_schema is not None
            else "OrderingRequest"
        )
        if request_schema is None:
            return create_model(name, ordering=(str | None, Field(default=None)))
        return create_model(
            name,
            __base__=request_schema,
            ordering=(str | None, Field(default=None)),
        )

    def order(self, items: Any, request: BaseModel) -> Any:
        raise NotImplementedError


class Ordering(BaseOrdering):
    def __init__(self, ordering_fields: list[str] | str = "__all__") -> None:
        self.ordering_fields = ordering_fields

    def order(self, items: Any, request: BaseModel) -> Any:
        raw_value = cast(str | None, getattr(request, "ordering", None))
        terms = self._parse_terms(raw_value)
        if not terms:
            return items

        valid_fields = set(self._valid_fields(items))
        if valid_fields:
            invalid = [term for term in terms if term.lstrip("-") not in valid_fields]
            if invalid:
                raise OrderingError(f"Invalid ordering fields: {', '.join(invalid)}")

        if isinstance(items, QuerySet):
            return items.order_by(*terms)

        if isinstance(items, list):
            return self._sort_list(items, terms)

        if isinstance(items, Iterable):
            return self._sort_list(list(items), terms)

        raise OrderingError(f"Result type '{type(items).__name__}' is not orderable.")

    def _sort_list(self, items: list[Any], terms: list[str]) -> list[Any]:
        if not items:
            return items
        is_dict = isinstance(items[0], dict)
        for term in reversed(terms):
            field_name = term.lstrip("-")
            reverse = term.startswith("-")
            getter = itemgetter(field_name) if is_dict else attrgetter(field_name)
            try:
                items.sort(key=getter, reverse=reverse)
            except Exception as exc:
                raise OrderingError(
                    f"Ordering field '{field_name}' is not available on list items."
                ) from exc
        return items

    def _parse_terms(self, value: str | None) -> list[str]:
        if not value:
            return []
        return [item.strip() for item in value.split(",") if item.strip()]

    def _valid_fields(self, items: Any) -> list[str]:
        if self.ordering_fields != "__all__":
            return list(cast(list[str], self.ordering_fields))
        if isinstance(items, QuerySet):
            return [str(field.name) for field in items.model._meta.fields] + [
                str(key) for key in items.query.annotations
            ]
        if isinstance(items, list) and items:
            first = items[0]
            if isinstance(first, BaseModel):
                return list(first.__class__.model_fields.keys())
            if isinstance(first, dict):
                return list(first.keys())
            if hasattr(first, "_meta") and hasattr(first._meta, "fields"):
                return [str(field.name) for field in first._meta.fields]
            attrs = [
                name
                for name in dir(first)
                if not name.startswith("_") and not callable(getattr(first, name))
            ]
            return attrs
        return []


def resolve_ordering_class(value: object | None) -> type[BaseOrdering] | None:
    if value is None:
        return None
    if isinstance(value, BaseOrdering):
        raise OrderingError(
            "Ordering instance is not supported here. "
            "Pass ordering class (e.g. Ordering) to `list_ordering_class` and "
            "pass fields via `list_ordering_fields`."
        )
    if isinstance(value, str):
        module_path, _, attr = value.rpartition(".")
        if not module_path or not attr:
            raise OrderingError(f"Invalid ordering class path: {value}")
        module = importlib.import_module(module_path)
        resolved = getattr(module, attr, None)
        if resolved is None:
            raise OrderingError(f"Ordering class '{value}' was not found.")
        value = resolved
    if not isinstance(value, type) or not issubclass(value, BaseOrdering):
        raise OrderingError("Ordering class must inherit BaseOrdering.")
    return value


def get_default_ordering_class() -> type[BaseOrdering] | None:
    configured = getattr(django_settings, "GRPC_EXTRA", {}) or {}
    value = configured.get("DEFAULT_ORDERING_CLASS", "grpc_extra.ordering.Ordering")
    return resolve_ordering_class(value)



from __future__ import annotations

import importlib
from collections.abc import Iterable
from typing import TYPE_CHECKING, Any, cast

if TYPE_CHECKING:
    from django.db.models import Model

    from .registry import MethodMeta


class PermissionError(Exception):
    pass


class BasePermission:
    """Base permission contract for gRPC service and method checks."""

    message = "Permission denied."

    def has_perm(
        self,
        request: Any,
        context: Any,
        service: object,
        method_meta: "MethodMeta",
    ) -> bool:
        return True

    def has_obj_perm(
        self,
        request: Any,
        context: Any,
        service: object,
        method_meta: "MethodMeta",
        obj: Any,
    ) -> bool:
        return True


PermissionLike = BasePermission | type[BasePermission] | str


class AllowAny(BasePermission):
    pass


class IsAuthenticated(BasePermission):
    message = "Authentication required."

    def has_perm(
        self,
        request: Any,
        context: Any,
        service: object,
        method_meta: "MethodMeta",
    ) -> bool:
        user = _resolve_user(context)
        if user is None:
            return False
        authenticated = getattr(user, "is_authenticated", None)
        if authenticated is None:
            return bool(user)
        return bool(authenticated)


class IsAuthenticatedActive(IsAuthenticated):
    message = "Active authenticated user required."

    def has_perm(
        self,
        request: Any,
        context: Any,
        service: object,
        method_meta: "MethodMeta",
    ) -> bool:
        if not super().has_perm(request, context, service, method_meta):
            return False
        user = _resolve_user(context)
        if user is None:
            return False
        is_active = getattr(user, "is_active", None)
        if is_active is None:
            return True
        return bool(is_active)


class IsAuthActive(IsAuthenticatedActive):
    """Alias for compatibility with existing naming habits."""


def _resolve_user(context: Any) -> Any | None:
    user = getattr(context, "user", None)
    if user is not None:
        return user
    return getattr(context, "auth", None)


class DjangoModelPermissions(BasePermission):
    """Permission checker for RPC model methods using Django auth codenames."""

    message = "Model permissions are required."
    perms_map: dict[str, list[str]] = {
        "LIST": ["%(app_label)s.view_%(model_name)s"],
        "STREAM_LIST": ["%(app_label)s.view_%(model_name)s"],
        "DETAIL": ["%(app_label)s.view_%(model_name)s"],
        "GET": ["%(app_label)s.view_%(model_name)s"],
        "CREATE": ["%(app_label)s.add_%(model_name)s"],
        "UPDATE": ["%(app_label)s.change_%(model_name)s"],
        "PATCH": ["%(app_label)s.change_%(model_name)s"],
        "DELETE": ["%(app_label)s.delete_%(model_name)s"],
        "DESTROY": ["%(app_label)s.delete_%(model_name)s"],
    }

    def __init__(self, model: type["Model"] | None = None) -> None:
        self.model = model

    def get_required_permissions(
        self, action: str, model_cls: type["Model"]
    ) -> list[str]:
        kwargs: dict[str, str | None] = {
            "app_label": model_cls._meta.app_label,
            "model_name": model_cls._meta.model_name,
        }
        if action not in self.perms_map:
            raise PermissionError(f"Unsupported permission RPC action: {action}")
        return [perm % kwargs for perm in self.perms_map[action]]

    def has_perm(
        self,
        request: Any,
        context: Any,
        service: object,
        method_meta: "MethodMeta",
    ) -> bool:
        user = _resolve_user(context)
        if user is None:
            return False
        has_perms = getattr(user, "has_perms", None)
        if not callable(has_perms):
            return False
        model_cls = self._resolve_model(service)
        if model_cls is None:
            return True
        action = self._resolve_action(method_meta)
        if action is None:
            return True
        perms = self.get_required_permissions(action, model_cls)
        return bool(has_perms(perms))

    def _resolve_model(self, service: object) -> type["Model"] | None:
        if self.model is not None:
            return self.model
        config = getattr(service, "config", None)
        if config is None:
            return None
        return cast(type["Model"] | None, getattr(config, "model", None))

    def _resolve_action(self, method_meta: "MethodMeta") -> str | None:
        candidate = method_meta.name.replace("-", "_").upper()
        if candidate in self.perms_map:
            return candidate
        fallback = method_meta.handler_name.replace("-", "_").upper()
        if fallback in self.perms_map:
            return fallback
        return None


def resolve_permission(value: PermissionLike) -> BasePermission:
    if isinstance(value, BasePermission):
        return value
    if isinstance(value, str):
        module_path, _, attr = value.rpartition(".")
        if not module_path or not attr:
            raise PermissionError(f"Invalid permission class path: {value}")
        module = importlib.import_module(module_path)
        resolved = getattr(module, attr, None)
        if not isinstance(resolved, type) or not issubclass(resolved, BasePermission):
            raise PermissionError(f"Permission '{value}' is not a BasePermission class")
        return resolved()
    if isinstance(value, type) and issubclass(value, BasePermission):
        return value()
    raise PermissionError("Permission must be instance, class or import path string.")


def resolve_permissions(
    values: Iterable[PermissionLike] | None,
) -> tuple[BasePermission, ...]:
    if values is None:
        return ()
    return tuple(resolve_permission(value) for value in values)



from __future__ import annotations

import importlib
from abc import ABC, abstractmethod
from collections.abc import Iterable
from typing import Any

from django.conf import settings as django_settings
from django.db.models import QuerySet
from pydantic import BaseModel, Field, create_model


class PaginationError(Exception):
    pass


class BasePagination(ABC):
    """Base class for gRPC pagination adapters."""

    @classmethod
    @abstractmethod
    def build_request_schema(
        cls, request_schema: type[BaseModel] | None
    ) -> type[BaseModel]:
        raise NotImplementedError

    @classmethod
    @abstractmethod
    def build_response_schema(cls, response_schema: type[BaseModel]) -> type[BaseModel]:
        raise NotImplementedError

    @classmethod
    @abstractmethod
    def paginate(cls, result: Any, request: BaseModel) -> dict[str, Any]:
        raise NotImplementedError


class LimitOffsetPagination(BasePagination):
    """Default limit/offset pagination for unary list endpoints."""

    default_limit = 100
    max_limit = 1000

    @classmethod
    def build_request_schema(
        cls, request_schema: type[BaseModel] | None
    ) -> type[BaseModel]:
        name = (
            f"{request_schema.__name__}WithLimitOffset"
            if request_schema is not None
            else "LimitOffsetRequest"
        )
        if request_schema is None:
            return create_model(
                name,
                limit=(int, Field(default=cls.default_limit, ge=1)),
                offset=(int, Field(default=0, ge=0)),
            )
        return create_model(
            name,
            __base__=request_schema,
            limit=(int, Field(default=cls.default_limit, ge=1)),
            offset=(int, Field(default=0, ge=0)),
        )

    @classmethod
    def build_response_schema(cls, response_schema: type[BaseModel]) -> type[BaseModel]:
        return create_model(
            f"{response_schema.__name__}Paginated",
            count=(int, ...),
            limit=(int, ...),
            offset=(int, ...),
            results=(list[response_schema], ...),  # type: ignore[valid-type]
        )

    @classmethod
    def paginate(cls, result: Any, request: BaseModel) -> dict[str, Any]:
        limit = min(
            max(int(getattr(request, "limit", cls.default_limit)), 1), cls.max_limit
        )
        offset = max(int(getattr(request, "offset", 0)), 0)

        if isinstance(result, QuerySet):
            count = result.count()
            page_items = list(result[offset : offset + limit])
            return {
                "count": count,
                "limit": limit,
                "offset": offset,
                "results": page_items,
            }

        if isinstance(result, Iterable):
            items = list(result)
            count = len(items)
            return {
                "count": count,
                "limit": limit,
                "offset": offset,
                "results": items[offset : offset + limit],
            }

        raise PaginationError(
            f"Result type '{type(result).__name__}' is not paginatable."
        )


def resolve_pagination_class(value: object | None) -> type[BasePagination] | None:
    if value is None:
        return None
    if isinstance(value, str):
        module_path, _, attr = value.rpartition(".")
        if not module_path or not attr:
            raise PaginationError(f"Invalid pagination class path: {value}")
        module = importlib.import_module(module_path)
        resolved = getattr(module, attr, None)
        if resolved is None:
            raise PaginationError(f"Pagination class '{value}' was not found.")
        value = resolved
    if not isinstance(value, type) or not issubclass(value, BasePagination):
        raise PaginationError("Pagination class must inherit BasePagination.")
    return value


def get_default_pagination_class() -> type[BasePagination] | None:
    configured = getattr(django_settings, "GRPC_EXTRA", {}) or {}
    value = configured.get(
        "DEFAULT_PAGINATION_CLASS",
        "grpc_extra.pagination.LimitOffsetPagination",
    )
    return resolve_pagination_class(value)



from __future__ import annotations

import importlib
import logging
import os
from typing import Iterable

import django
from django.apps import apps

from .adapters import ServiceRuntimeAdapter
from .auth import AuthInterceptor, resolve_auth_backend
from .constants import GRPC_SERVICE_META
from .registry import ServiceDefinition, ServiceNotDecoratedError, registry
from .request_logging import GrpcRequestLoggingInterceptor
from .settings import get_grpc_extra_settings
from .utils import pb2_grpc_module_path, pb2_module_path

logger = logging.getLogger("grpc_extra")


def _run_reloaded_server(pattern: str, kwargs: dict) -> None:
    _ensure_django_ready()
    GrpcExtra(pattern=pattern).run_server(reload=False, **kwargs)


def _ensure_django_ready() -> None:
    if apps.ready:
        return
    if not os.environ.get("DJANGO_SETTINGS_MODULE"):
        raise ServiceNotDecoratedError(
            "DJANGO_SETTINGS_MODULE is not set in reload process."
        )
    django.setup()


class GrpcExtra:
    """Entry point for gRPC runtime integration in a Django project.

    `GrpcExtra` is responsible for the full service lifecycle at runtime:

    1. Discover decorated services from Django apps (`auto_discover_services`).
    2. Load generated protobuf modules for each discovered service.
    3. Adapt business methods to gRPC handlers (request/response conversion,
       streaming wrappers, exception mapping).
    4. Register servicers on a gRPC server (`apply`) and optionally run the
       server loop (`run_server`).

    The class expects services to be declared with `@grpc_service` and methods
    with `@grpc_method`. Proto and pb2 modules are resolved using service
    metadata (`app_label`, `proto_path`, `name`) collected by decorators.
    """

    def __init__(self, pattern: str = "{app}.grpc.services") -> None:
        self.pattern = pattern

    def register_service(self, service: type) -> ServiceDefinition:
        return registry.register(service)

    def auto_discover_services(self) -> "GrpcExtra":
        logger.debug("grpc service autodiscovery started pattern=%s", self.pattern)
        for app_conf in apps.get_app_configs():
            module_path = self.pattern.format(
                app=app_conf.name,
                label=app_conf.label,
            )
            module = self._safe_import(module_path)
            if module is None:
                continue
            for obj in self._iter_decorated_services(module):
                self.register_service(obj)
                logger.debug(
                    "grpc service discovered app=%s service=%s",
                    app_conf.label,
                    getattr(obj, "__name__", "<unknown>"),
                )

        logger.debug("grpc service autodiscovery completed")
        return self

    def apply(self, server) -> list[str]:
        """Register all discovered services on the provided gRPC server."""
        registered: list[str] = []
        settings = get_grpc_extra_settings()
        logger.debug("grpc service apply started")
        for definition in registry.all():
            meta = definition.meta
            if not meta.proto_path:
                raise ServiceNotDecoratedError(
                    f"Service '{meta.name}' must define proto_path."
                )
            pb2_grpc_path = pb2_grpc_module_path(meta.app_label, meta.proto_path)
            pb2_path = pb2_module_path(meta.app_label, meta.proto_path)
            try:
                pb2_grpc = importlib.import_module(pb2_grpc_path)
                pb2 = importlib.import_module(pb2_path)
            except ModuleNotFoundError as exc:
                raise ServiceNotDecoratedError(
                    "pb2_grpc module not found. "
                    f"Expected '{pb2_grpc_path}'. "
                    "Generate proto files before starting the server."
                ) from exc

            add_name = f"add_{meta.name}Servicer_to_server"
            add_servicer = getattr(pb2_grpc, add_name, None)
            if not callable(add_servicer):
                raise ServiceNotDecoratedError(
                    f"Servicer registration function '{add_name}' not found "
                    f"in '{pb2_grpc_path}'."
                )

            factory = meta.factory or definition.service
            servicer = factory()
            ServiceRuntimeAdapter(
                definition,
                pb2,
                exception_mapper=settings.exception_mapper,
            ).apply(servicer)
            add_servicer(servicer, server)
            registered.append(definition.meta.name)
            logger.info("grpc service registered service=%s", definition.meta.name)
        logger.debug("grpc service apply completed count=%s", len(registered))
        return registered

    def run_server(
        self,
        *,
        bind: str | None = None,
        max_workers: int | None = None,
        max_msg_mb: int | None = None,
        enable_health: bool | None = None,
        enable_reflection: bool | None = None,
        auth_backend=None,
        interceptors: Iterable[object] | None = None,
        reload: bool | None = None,
        reload_paths: Iterable[str] | None = None,
        auto_discover: bool = True,
    ) -> None:
        """Start gRPC server and optionally restart process on file changes."""
        from concurrent import futures

        import grpc

        settings = get_grpc_extra_settings()
        bind = bind or settings.bind
        max_workers = max_workers or settings.max_workers
        max_msg_mb = max_msg_mb or settings.max_msg_mb
        enable_health = (
            settings.enable_health if enable_health is None else enable_health
        )
        enable_reflection = (
            settings.enable_reflection
            if enable_reflection is None
            else enable_reflection
        )
        reload = settings.enable_reload if reload is None else reload
        reload_paths = (
            tuple(settings.reload_paths)
            if reload_paths is None
            else tuple(reload_paths)
        )
        auth_backend = settings.auth_backend if auth_backend is None else auth_backend
        interceptors = (
            list(settings.interceptors) if interceptors is None else list(interceptors)
        )
        logger_name = settings.logger_name

        if reload:
            if interceptors:
                raise ServiceNotDecoratedError(
                    "Live reload mode does not support runtime interceptor objects. "
                    "Configure importable interceptors in Django settings."
                )
            self._run_with_reload(
                bind=bind,
                max_workers=max_workers,
                max_msg_mb=max_msg_mb,
                enable_health=enable_health,
                enable_reflection=enable_reflection,
                auth_backend=auth_backend,
                reload_paths=reload_paths,
                auto_discover=auto_discover,
            )
            return

        if auto_discover:
            self.auto_discover_services()

        options = [
            ("grpc.max_send_message_length", max_msg_mb * 1024 * 1024),
            ("grpc.max_receive_message_length", max_msg_mb * 1024 * 1024),
        ]

        resolved_backend = resolve_auth_backend(auth_backend)
        if resolved_backend is not None:
            interceptors.append(AuthInterceptor(resolved_backend))
            logger.debug("grpc auth interceptor enabled")
        if settings.enable_request_logging:
            interceptors.append(GrpcRequestLoggingInterceptor(logger_name=logger_name))
            logger.debug(
                "grpc request logging interceptor enabled logger=%s", logger_name
            )

        server = grpc.server(
            futures.ThreadPoolExecutor(max_workers=max_workers),
            options=options,
            interceptors=interceptors,
        )
        self.apply(server)

        service_names = [
            self._qualified_service_name(definition) for definition in registry.all()
        ]
        if enable_health:
            self._add_health(server)
        if enable_reflection:
            self._add_reflection(server, service_names, enable_health=enable_health)

        server.add_insecure_port(bind)
        server.start()
        logger.info("grpc server started bind=%s", bind)
        try:
            server.wait_for_termination()
        except KeyboardInterrupt:
            # Reload workers are stopped via signal; avoid noisy tracebacks.
            logger.debug("grpc server termination requested")
            server.stop(grace=0)

    def _run_with_reload(
        self,
        *,
        bind: str,
        max_workers: int,
        max_msg_mb: int,
        enable_health: bool,
        enable_reflection: bool,
        auth_backend,
        reload_paths: tuple[str, ...],
        auto_discover: bool,
    ) -> None:
        try:
            from watchfiles import run_process
        except ImportError as exc:
            raise ServiceNotDecoratedError(
                "watchfiles is required for live reload mode. "
                "Install `django-grpc-extra[reload]`."
            ) from exc

        kwargs = {
            "bind": bind,
            "max_workers": max_workers,
            "max_msg_mb": max_msg_mb,
            "enable_health": enable_health,
            "enable_reflection": enable_reflection,
            "auth_backend": auth_backend,
            "auto_discover": auto_discover,
        }
        run_process(
            *reload_paths,
            target=_run_reloaded_server,
            args=(self.pattern, kwargs),
        )

    @staticmethod
    def _add_health(server) -> None:
        try:
            from grpc_health.v1 import health, health_pb2_grpc
        except ImportError as exc:
            raise ServiceNotDecoratedError(
                "grpcio-health is required for health checks."
            ) from exc
        health_servicer = health.HealthServicer()
        health_pb2_grpc.add_HealthServicer_to_server(health_servicer, server)

    @staticmethod
    def _add_reflection(
        server, service_names: Iterable[str], *, enable_health: bool
    ) -> None:
        try:
            from grpc_reflection.v1alpha import reflection
        except ImportError as exc:
            raise ServiceNotDecoratedError(
                "grpcio-reflection is required for reflection."
            ) from exc
        names = list(service_names)
        if enable_health:
            names.append("grpc.health.v1.Health")
        names.append(reflection.SERVICE_NAME)
        reflection.enable_server_reflection(names, server)

    @staticmethod
    def _qualified_service_name(definition: ServiceDefinition) -> str:
        package = definition.meta.package
        if package:
            return f"{package}.{definition.meta.name}"
        return definition.meta.name

    @staticmethod
    def _safe_import(module_path: str):
        try:
            return importlib.import_module(module_path)
        except ModuleNotFoundError as exc:
            if exc.name and module_path.startswith(exc.name):
                return None
            if exc.name == module_path:
                return None
            raise

    @staticmethod
    def _iter_decorated_services(module) -> Iterable[type]:
        for attr_name in dir(module):
            obj = getattr(module, attr_name)
            if getattr(obj, GRPC_SERVICE_META, None) is not None:
                yield obj



from __future__ import annotations

import logging
import time

import grpc


class GrpcRequestLoggingInterceptor(grpc.ServerInterceptor):
    """Logs incoming grpc calls and execution time using Django logging config."""

    def __init__(self, logger_name: str = "grpc_extra") -> None:
        self.logger = logging.getLogger(logger_name)

    def intercept_service(self, continuation, handler_call_details):
        handler = continuation(handler_call_details)
        if handler is None:
            return None

        method = handler_call_details.method

        if handler.unary_unary:

            def unary_unary(request, context):
                return self._run(method, context, handler.unary_unary, request)

            return grpc.unary_unary_rpc_method_handler(
                unary_unary,
                request_deserializer=handler.request_deserializer,
                response_serializer=handler.response_serializer,
            )

        if handler.unary_stream:

            def unary_stream(request, context):
                return self._run(method, context, handler.unary_stream, request)

            return grpc.unary_stream_rpc_method_handler(
                unary_stream,
                request_deserializer=handler.request_deserializer,
                response_serializer=handler.response_serializer,
            )

        if handler.stream_unary:

            def stream_unary(request_iterator, context):
                return self._run(
                    method, context, handler.stream_unary, request_iterator
                )

            return grpc.stream_unary_rpc_method_handler(
                stream_unary,
                request_deserializer=handler.request_deserializer,
                response_serializer=handler.response_serializer,
            )

        if handler.stream_stream:

            def stream_stream(request_iterator, context):
                return self._run(
                    method, context, handler.stream_stream, request_iterator
                )

            return grpc.stream_stream_rpc_method_handler(
                stream_stream,
                request_deserializer=handler.request_deserializer,
                response_serializer=handler.response_serializer,
            )

        return handler

    def _run(self, method: str, context, fn, request_or_iterator):
        started = time.perf_counter()
        self.logger.debug("grpc request started method=%s", method)
        try:
            return fn(request_or_iterator, context)
        finally:
            duration_ms = (time.perf_counter() - started) * 1000
            self.logger.info(
                "grpc request finished method=%s duration_ms=%.2f", method, duration_ms
            )



from __future__ import annotations

from abc import ABC, abstractmethod

from django.db.models import QuerySet
from pydantic import BaseModel
from typing import cast

from .filtering import ModelFilterSchema
from .schemas import ModelServiceConfig


class ModelDataHelper(ABC):
    """Abstraction for model CRUD operations used by ModelService."""

    def __init__(self, config: ModelServiceConfig) -> None:
        self.config = config

    @abstractmethod
    def list_objects(self, request: BaseModel | None) -> QuerySet:
        raise NotImplementedError

    @abstractmethod
    def get_object(self, request: BaseModel):
        raise NotImplementedError

    @abstractmethod
    def create_object(self, request: BaseModel):
        raise NotImplementedError

    @abstractmethod
    def update_object(self, request: BaseModel):
        raise NotImplementedError

    @abstractmethod
    def patch_object(self, request: BaseModel):
        raise NotImplementedError

    @abstractmethod
    def delete_object(self, request: BaseModel):
        raise NotImplementedError


class DefaultModelDataHelper(ModelDataHelper):
    """Default CRUD implementation backed by Django ORM."""

    def get_queryset(self) -> QuerySet:
        return self._resolve_queryset(self.config.queryset, option_name="queryset")

    def get_detail_queryset(self) -> QuerySet:
        configured = self.config.detail_queryset
        if configured is None:
            return self.get_queryset()
        return self._resolve_queryset(configured, option_name="detail_queryset")

    def _resolve_queryset(self, configured, *, option_name: str) -> QuerySet:
        if configured is None:
            return self.config.model.objects.all()
        queryset = configured() if callable(configured) else configured
        if queryset is None or not hasattr(queryset, "all"):
            raise TypeError(
                f"ModelServiceConfig.{option_name} must be a QuerySet-like object or callable returning it."
            )
        return cast(QuerySet, queryset.all())

    def get_lookup_value(self, request: BaseModel):
        return getattr(request, self.config.lookup_field)

    def list_objects(self, request: BaseModel | None) -> QuerySet:
        queryset = self.get_queryset()
        filter_schema = self.config.list_filter
        if request is None or filter_schema is None:
            return queryset

        filter_payload = {
            field_name: getattr(request, field_name, None)
            for field_name in filter_schema.model_fields
        }
        filter_request = filter_schema.model_validate(filter_payload)
        if isinstance(filter_request, ModelFilterSchema):
            return cast(QuerySet, filter_request.filter_queryset(queryset))

        filter_kwargs = {
            key: value for key, value in filter_payload.items() if value is not None
        }
        if not filter_kwargs:
            return queryset
        return queryset.filter(**filter_kwargs)

    def get_object(self, request: BaseModel):
        lookup_value = self.get_lookup_value(request)
        return self.get_detail_queryset().get(
            **{self.config.lookup_field: lookup_value}
        )

    def create_object(self, request: BaseModel):
        payload = request.model_dump(mode="python", by_alias=True, exclude_none=True)
        return self.config.model.objects.create(**payload)

    def update_object(self, request: BaseModel):
        instance = self.get_queryset().get(
            **{self.config.lookup_field: self.get_lookup_value(request)}
        )
        payload_model = getattr(request, "payload")
        payload = payload_model.model_dump(
            mode="python",
            by_alias=True,
            exclude_none=True,
            exclude_unset=False,
        )
        for key, value in payload.items():
            setattr(instance, key, value)
        instance.save()
        return instance

    def patch_object(self, request: BaseModel):
        instance = self.get_queryset().get(
            **{self.config.lookup_field: self.get_lookup_value(request)}
        )
        payload_model = getattr(request, "payload")
        payload = payload_model.model_dump(
            mode="python",
            by_alias=True,
            exclude_none=True,
            exclude_unset=True,
        )
        for key, value in payload.items():
            setattr(instance, key, value)
        instance.save()
        return instance

    def delete_object(self, request: BaseModel):
        instance = self.get_queryset().get(
            **{self.config.lookup_field: self.get_lookup_value(request)}
        )
        instance.delete()
        return {}



from __future__ import annotations

import importlib
import inspect
from collections.abc import Callable
from dataclasses import dataclass
from typing import cast

import grpc
from django.core.exceptions import ObjectDoesNotExist
from pydantic import ValidationError

from .ordering import OrderingError
from .searching import SearchingError


class GrpcExtraError(Exception):
    """Base error for grpc_extra runtime conversion and adapter failures."""


class RequestDecodeError(GrpcExtraError):
    """Raised when grpc request payload cannot be decoded into expected schema."""


class ResponseEncodeError(GrpcExtraError):
    """Raised when business result cannot be encoded into grpc response payload."""


@dataclass(frozen=True)
class MappedError:
    code: grpc.StatusCode
    message: str


def default_exception_mapper(exc: Exception) -> MappedError:
    if isinstance(exc, ValidationError):
        return MappedError(grpc.StatusCode.INVALID_ARGUMENT, str(exc))
    if isinstance(exc, RequestDecodeError) and isinstance(
        exc.__cause__, ValidationError
    ):
        return MappedError(grpc.StatusCode.INVALID_ARGUMENT, str(exc.__cause__))
    if isinstance(exc, ObjectDoesNotExist):
        return MappedError(grpc.StatusCode.NOT_FOUND, str(exc))
    if isinstance(exc, PermissionError):
        return MappedError(grpc.StatusCode.PERMISSION_DENIED, str(exc))
    if isinstance(exc, (OrderingError, SearchingError)):
        return MappedError(grpc.StatusCode.INVALID_ARGUMENT, str(exc))
    if isinstance(exc, (RequestDecodeError, ResponseEncodeError)):
        return MappedError(grpc.StatusCode.INTERNAL, str(exc))
    return MappedError(grpc.StatusCode.UNKNOWN, str(exc))


def resolve_exception_mapper(
    mapper: Callable[[Exception], MappedError] | str | None,
) -> Callable[[Exception], MappedError]:
    if mapper is None:
        return default_exception_mapper
    if isinstance(mapper, str):
        module_path, _, attr = mapper.rpartition(".")
        if not module_path or not attr:
            raise ValueError(f"Invalid exception mapper path: {mapper}")
        module = importlib.import_module(module_path)
        resolved = getattr(module, attr, None)
        if not callable(resolved) or inspect.isclass(resolved):
            raise ValueError(f"Exception mapper '{mapper}' is not callable")
        return cast(Callable[[Exception], MappedError], resolved)
    if inspect.isclass(mapper):
        raise ValueError("Exception mapper must be a function/callable, not a class.")
    return mapper



def is_upper_camel_case(s: str) -> bool:
    return s != "" and s[0].isupper() and not any(c in s for c in "_-")


def to_upper_camel_case(value: str) -> str:
    normalized = value.replace("-", "_")
    if "_" in normalized:
        parts = [part for part in normalized.split("_") if part]
        return "".join(part[:1].upper() + part[1:] for part in parts)
    return normalized[:1].upper() + normalized[1:]


def normalize_proto_path(proto_path: str) -> str:
    normalized = proto_path.replace("\\", "/").lstrip("/")
    if not normalized.endswith(".proto"):
        normalized = f"{normalized}.proto"
    return normalized


def proto_path_to_module(app_label: str, proto_path: str) -> str:
    normalized = normalize_proto_path(proto_path)
    without_ext = normalized[:-6]
    module_rel = without_ext.replace("/", ".")
    return f"{app_label}.{module_rel}"


def pb2_module_path(app_label: str, proto_path: str) -> str:
    return f"{proto_path_to_module(app_label, proto_path)}_pb2"


def pb2_grpc_module_path(app_label: str, proto_path: str) -> str:
    return f"{proto_path_to_module(app_label, proto_path)}_pb2_grpc"



GET = "Get"
LIST = "List"
CREATE = "Create"
UPDATE = "Update"
PATCH = "Patch"
DELETE = "Delete"

GRPC_SERVICE_META = "__grpc_service_meta__"
GRPC_METHOD_META = "__grpc_method_meta__"
GRPC_PAGINATION_META = "__grpc_pagination_meta__"
GRPC_ORDERING_META = "__grpc_ordering_meta__"
GRPC_SEARCHING_META = "__grpc_searching_meta__"
GRPC_PERMISSIONS_META = "__grpc_permissions_meta__"



from __future__ import annotations

import re
import shutil
import subprocess
import textwrap
from dataclasses import dataclass
from pathlib import Path
from typing import Iterable

from django.conf import settings as django_settings


class SDKGenerationError(Exception):
    pass


class BaseClientSDKGenerator:
    language: str

    def generate(
        self,
        *,
        proto_files: Iterable[Path],
        out_dir: Path,
        sdk_name: str,
        include_root: Path,
    ) -> Path:
        raise NotImplementedError


@dataclass
class RpcMethodSpec:
    name: str
    python_name: str
    request_type: str
    response_type: str
    client_streaming: bool
    server_streaming: bool


@dataclass
class ServiceSpec:
    sdk_app: str
    service_name: str
    python_attr: str
    class_name: str
    stub_module: str
    stub_class: str
    stub_alias: str
    pb2_module: str
    pb2_alias: str
    methods: list[RpcMethodSpec]


@dataclass
class MessageFieldSpec:
    name: str
    annotation: str
    default_expr: str


@dataclass
class MessageSpec:
    sdk_app: str
    pb2_alias: str
    message_name: str
    model_name: str
    fields: list[MessageFieldSpec]


class PythonClientSDKGenerator(BaseClientSDKGenerator):
    language = "python"

    def generate(
        self,
        *,
        proto_files: Iterable[Path],
        out_dir: Path,
        sdk_name: str,
        include_root: Path,
    ) -> Path:
        try:
            import grpc_tools
            from grpc_tools import protoc
        except ImportError as exc:
            raise SDKGenerationError(
                "grpcio-tools is required for Python SDK generation."
            ) from exc

        proto_list = [Path(p) for p in proto_files]
        if not proto_list:
            raise SDKGenerationError(
                "No proto files provided for Python SDK generation."
            )

        target_dir = out_dir / sdk_name
        target_exists = target_dir.exists()
        src_dir = target_dir / "src"
        package_name = self._package_name(sdk_name)
        runtime_dir = src_dir / package_name

        target_dir.mkdir(parents=True, exist_ok=True)
        src_dir.mkdir(parents=True, exist_ok=True)
        runtime_dir.mkdir(parents=True, exist_ok=True)

        grpc_tools_include = self._grpc_tools_include_path(grpc_tools)
        for proto_file in proto_list:
            args = [
                "protoc",
                f"-I{include_root}",
                f"--python_out={src_dir}",
                f"--grpc_python_out={src_dir}",
                f"--pyi_out={src_dir}",
            ]
            if grpc_tools_include is not None:
                args.append(f"-I{grpc_tools_include}")
            args.append(str(proto_file))
            result = protoc.main(args)
            if result != 0:
                raise SDKGenerationError(f"protoc failed for {proto_file}")
            self._ensure_python_package_tree(src_dir, proto_file, include_root)

        service_specs = self._collect_service_specs(proto_list, include_root)
        message_specs = self._collect_message_specs(proto_list, include_root)
        model_name_map = {
            (spec.pb2_alias, spec.message_name): spec.model_name
            for spec in message_specs
        }
        services_by_app = self._group_services_by_app(service_specs)
        messages_by_app = self._group_messages_by_app(message_specs)

        generated_dir = runtime_dir / "generated"
        generated_dir.mkdir(parents=True, exist_ok=True)
        (generated_dir / "__init__.py").write_text("__all__ = []\n", encoding="utf-8")
        (runtime_dir / "_service_base.py").write_text(
            self._render_service_base(), encoding="utf-8"
        )
        (runtime_dir / "_typed_base.py").write_text(
            self._render_typed_base(), encoding="utf-8"
        )

        all_apps = sorted(set(services_by_app) | set(messages_by_app))
        for app in all_apps:
            app_dir = generated_dir / app
            app_dir.mkdir(parents=True, exist_ok=True)
            (app_dir / "__init__.py").write_text("__all__ = []\n", encoding="utf-8")
            (app_dir / "services.py").write_text(
                self._render_app_services(services_by_app.get(app, [])),
                encoding="utf-8",
            )
            (app_dir / "models.py").write_text(
                self._render_app_models(messages_by_app.get(app, [])),
                encoding="utf-8",
            )
            (app_dir / "typed_services.py").write_text(
                self._render_app_typed_services(
                    services_by_app.get(app, []), model_name_map
                ),
                encoding="utf-8",
            )

        (runtime_dir / "services.py").write_text(
            self._render_services_index(services_by_app), encoding="utf-8"
        )
        (runtime_dir / "models.py").write_text(
            self._render_models_index(messages_by_app), encoding="utf-8"
        )
        (runtime_dir / "typed_services.py").write_text(
            self._render_typed_services_index(services_by_app), encoding="utf-8"
        )
        (runtime_dir / "client_generated.py").write_text(
            self._render_generated_client(service_specs), encoding="utf-8"
        )
        if not target_exists:
            (runtime_dir / "__init__.py").write_text(
                self._render_init(package_name, service_specs), encoding="utf-8"
            )
            (runtime_dir / "auth.py").write_text(self._render_auth(), encoding="utf-8")
            (runtime_dir / "config.py").write_text(
                self._render_config(package_name), encoding="utf-8"
            )
            (runtime_dir / "errors.py").write_text(
                self._render_errors(), encoding="utf-8"
            )
            (runtime_dir / "client.py").write_text(
                self._render_client_wrapper(), encoding="utf-8"
            )
            (runtime_dir / "helpers.py").write_text(
                self._render_helpers(), encoding="utf-8"
            )
        else:
            client_py = runtime_dir / "client.py"
            if not client_py.exists():
                client_py.write_text(self._render_client_wrapper(), encoding="utf-8")
            helpers_py = runtime_dir / "helpers.py"
            if not helpers_py.exists():
                helpers_py.write_text(self._render_helpers(), encoding="utf-8")

        if not target_exists:
            (target_dir / "pyproject.toml").write_text(
                self._render_pyproject(sdk_name, package_name), encoding="utf-8"
            )
            (target_dir / "README.md").write_text(
                self._render_readme(sdk_name, package_name), encoding="utf-8"
            )

        return target_dir

    def _package_name(self, sdk_name: str) -> str:
        sanitized = re.sub(r"[^a-zA-Z0-9]+", "_", sdk_name).strip("_").lower()
        return sanitized or "grpc_client_sdk"

    def _grpc_tools_include_path(self, grpc_tools_module: object) -> Path | None:
        module_file = getattr(grpc_tools_module, "__file__", None)
        if not module_file:
            return None
        include_path = Path(module_file).resolve().parent / "_proto"
        if include_path.exists():
            return include_path
        return None

    def _ensure_python_package_tree(
        self,
        src_dir: Path,
        proto_file: Path,
        include_root: Path,
    ) -> None:
        rel = proto_file.relative_to(include_root)
        generated_file = src_dir / f"{rel.with_suffix('')}_pb2.py"
        current = generated_file.parent
        while current != src_dir.parent:
            current.mkdir(parents=True, exist_ok=True)
            init_file = current / "__init__.py"
            if not init_file.exists():
                init_file.touch()
            if current == src_dir:
                break
            current = current.parent

    def _collect_service_specs(
        self,
        proto_files: list[Path],
        include_root: Path,
    ) -> list[ServiceSpec]:
        specs: list[ServiceSpec] = []
        for proto_file in proto_files:
            proto_text = proto_file.read_text(encoding="utf-8")
            rel = proto_file.relative_to(include_root)
            sdk_app = self._sdk_app_name(rel)
            module_base = ".".join(rel.with_suffix("").parts)
            service_blocks = re.finditer(
                r"service\s+(?P<name>\w+)\s*\{(?P<body>.*?)\}",
                proto_text,
                flags=re.DOTALL,
            )
            for block in service_blocks:
                service_name = block.group("name")
                body = block.group("body")
                methods = self._parse_service_methods(body)
                if not methods:
                    continue
                specs.append(
                    ServiceSpec(
                        sdk_app=sdk_app,
                        service_name=service_name,
                        python_attr=self._to_snake(
                            service_name.removesuffix("Service") or service_name
                        ),
                        class_name=f"{service_name}Client",
                        stub_module=f"{module_base}_pb2_grpc",
                        stub_class=f"{service_name}Stub",
                        stub_alias=self._symbol_alias(
                            f"{module_base}_pb2_grpc", f"{service_name}Stub"
                        ),
                        pb2_module=f"{module_base}_pb2",
                        pb2_alias=self._module_alias(f"{module_base}_pb2"),
                        methods=methods,
                    )
                )
        return specs

    def _collect_message_specs(
        self,
        proto_files: list[Path],
        include_root: Path,
    ) -> list[MessageSpec]:
        raw_specs: list[tuple[str, str, str, list[MessageFieldSpec]]] = []
        for proto_file in proto_files:
            proto_text = proto_file.read_text(encoding="utf-8")
            rel = proto_file.relative_to(include_root)
            sdk_app = self._sdk_app_name(rel)
            module_base = ".".join(rel.with_suffix("").parts)
            pb2_alias = self._module_alias(f"{module_base}_pb2")
            for match in re.finditer(
                r"message\s+(?P<name>\w+)\s*\{(?P<body>.*?)\}",
                proto_text,
                flags=re.DOTALL,
            ):
                message_name = match.group("name")
                fields = self._parse_message_fields(match.group("body"), pb2_alias)
                raw_specs.append((sdk_app, pb2_alias, message_name, fields))

        counts: dict[str, int] = {}
        for _, _, message_name, _ in raw_specs:
            counts[message_name] = counts.get(message_name, 0) + 1

        specs: list[MessageSpec] = []
        for sdk_app, pb2_alias, message_name, fields in raw_specs:
            if counts[message_name] == 1:
                model_name = message_name
            else:
                model_name = f"{self._pascal_from_alias(pb2_alias)}_{message_name}"
            specs.append(
                MessageSpec(
                    sdk_app=sdk_app,
                    pb2_alias=pb2_alias,
                    message_name=message_name,
                    model_name=model_name,
                    fields=fields,
                )
            )
        return specs

    def _parse_message_fields(
        self, message_body: str, pb2_alias: str
    ) -> list[MessageFieldSpec]:
        fields: list[MessageFieldSpec] = []
        for raw_line in message_body.splitlines():
            line = raw_line.strip()
            if not line or line.startswith("//"):
                continue
            if "//" in line:
                line = line.split("//", 1)[0].strip()
            if not line.endswith(";"):
                continue
            match = re.match(
                r"(?:(optional|required|repeated)\s+)?(?P<typ>map<[^>]+>|[.\w]+)\s+(?P<name>\w+)\s*=\s*\d+",
                line,
            )
            if not match:
                continue
            label = match.group(1) or ""
            proto_type = match.group("typ")
            field_name = match.group("name")
            annotation, default_expr = self._python_field_type(
                proto_type=proto_type,
                label=label,
                pb2_alias=pb2_alias,
            )
            fields.append(
                MessageFieldSpec(
                    name=field_name,
                    annotation=annotation,
                    default_expr=default_expr,
                )
            )
        return fields

    def _python_field_type(
        self,
        *,
        proto_type: str,
        label: str,
        pb2_alias: str,
    ) -> tuple[str, str]:
        if proto_type.startswith("map<") and proto_type.endswith(">"):
            key_type, value_type = [
                part.strip() for part in proto_type[4:-1].split(",", 1)
            ]
            py_key, _ = self._python_field_type(
                proto_type=key_type, label="", pb2_alias=pb2_alias
            )
            py_value, _ = self._python_field_type(
                proto_type=value_type, label="", pb2_alias=pb2_alias
            )
            return (f"dict[{py_key}, {py_value}]", "Field(default_factory=dict)")

        scalar_map = {
            "double": "float",
            "float": "float",
            "int32": "int",
            "int64": "int",
            "uint32": "int",
            "uint64": "int",
            "sint32": "int",
            "sint64": "int",
            "fixed32": "int",
            "fixed64": "int",
            "sfixed32": "int",
            "sfixed64": "int",
            "bool": "bool",
            "string": "str",
            "bytes": "str",
            "google.protobuf.Timestamp": "datetime",
            "google.protobuf.Empty": "dict[str, Any]",
        }
        message_name = proto_type.split(".")[-1]
        base_type = scalar_map.get(proto_type)
        if base_type is None:
            if "." in proto_type and not proto_type.startswith("google.protobuf."):
                base_type = "Any"
            elif message_name and message_name[0].isupper():
                base_type = message_name
            else:
                base_type = "Any"

        if label == "repeated":
            return (f"list[{base_type}]", "Field(default_factory=list)")
        if label == "optional":
            return (f"{base_type} | None", "None")
        return (base_type, "...")

    def _pascal_from_alias(self, alias: str) -> str:
        parts = [part for part in alias.split("_") if part]
        return "".join(part[:1].upper() + part[1:] for part in parts)

    def _parse_service_methods(self, body: str) -> list[RpcMethodSpec]:
        methods: list[RpcMethodSpec] = []
        for match in re.finditer(
            r"rpc\s+(?P<name>\w+)\s*\("
            r"(?P<req_stream>stream\s+)?(?P<req>[.\w]+)\)\s*returns\s*\("
            r"(?P<res_stream>stream\s+)?(?P<res>[.\w]+)\)",
            body,
        ):
            method_name = match.group("name")
            methods.append(
                RpcMethodSpec(
                    name=method_name,
                    python_name=self._to_snake(method_name),
                    request_type=match.group("req"),
                    response_type=match.group("res"),
                    client_streaming=bool(match.group("req_stream")),
                    server_streaming=bool(match.group("res_stream")),
                )
            )
        return methods

    def _to_snake(self, value: str) -> str:
        first_pass = re.sub("(.)([A-Z][a-z]+)", r"\1_\2", value)
        second_pass = re.sub("([a-z0-9])([A-Z])", r"\1_\2", first_pass)
        return second_pass.lower()

    def _module_alias(self, module_path: str) -> str:
        return re.sub(r"[^a-zA-Z0-9]+", "_", module_path).strip("_")

    def _symbol_alias(self, module_path: str, symbol: str) -> str:
        alias = f"{module_path}.{symbol}"
        return re.sub(r"[^a-zA-Z0-9]+", "_", alias).strip("_")

    def _sdk_app_name(self, rel_proto_path: Path) -> str:
        first = rel_proto_path.parts[0] if rel_proto_path.parts else "default"
        sanitized = re.sub(r"[^a-zA-Z0-9]+", "_", first).strip("_").lower()
        return sanitized or "default"

    def _render_init(self, package_name: str, specs: list[ServiceSpec]) -> str:
        service_exports = [spec.class_name for spec in specs]
        exports = [
            "ClientConfig",
            "GrpcClient",
            "TypedGrpcClient",
            "AuthProvider",
            "StaticTokenAuth",
            "RefreshingTokenAuth",
            "message_to_dict",
            "extract_results",
            *service_exports,
        ]
        lines = [
            "from .auth import AuthProvider, RefreshingTokenAuth, StaticTokenAuth",
            "from .client import GrpcClient",
            "from .config import ClientConfig",
            "from .helpers import extract_results, message_to_dict",
            "from .typed_services import TypedGrpcClient",
            "from .services import " + ", ".join(service_exports)
            if service_exports
            else "",
            "",
            f"__all__ = {exports!r}",
        ]
        return "\n".join(line for line in lines if line)

    def _render_auth(self) -> str:
        return (
            textwrap.dedent(
                """
            from __future__ import annotations

            import threading
            import time
            from collections.abc import Callable, Sequence
            from typing import Protocol


            class AuthProvider(Protocol):
                def get_metadata(self) -> Sequence[tuple[str, str]]:
                    ...


            class StaticTokenAuth:
                def __init__(
                    self,
                    token: str,
                    *,
                    header: str = "authorization",
                    prefix: str = "Bearer",
                ) -> None:
                    self.token = token
                    self.header = header
                    self.prefix = prefix

                def get_metadata(self) -> Sequence[tuple[str, str]]:
                    value = f"{self.prefix} {self.token}".strip()
                    return ((self.header, value),)


            class RefreshingTokenAuth:
                def __init__(
                    self,
                    token_provider: Callable[[], str],
                    *,
                    refresh_interval_sec: float = 300.0,
                    header: str = "authorization",
                    prefix: str = "Bearer",
                ) -> None:
                    self.token_provider = token_provider
                    self.refresh_interval_sec = refresh_interval_sec
                    self.header = header
                    self.prefix = prefix
                    self._token: str | None = None
                    self._expires_at = 0.0
                    self._lock = threading.Lock()

                def get_metadata(self) -> Sequence[tuple[str, str]]:
                    token = self._current_token()
                    value = f"{self.prefix} {token}".strip()
                    return ((self.header, value),)

                def _current_token(self) -> str:
                    now = time.monotonic()
                    if self._token is not None and now < self._expires_at:
                        return self._token
                    with self._lock:
                        now = time.monotonic()
                        if self._token is None or now >= self._expires_at:
                            self._token = self.token_provider()
                            self._expires_at = now + self.refresh_interval_sec
                        return self._token
            """
            ).strip()
            + "\n"
        )

    def _render_config(self, package_name: str) -> str:
        return (
            textwrap.dedent(
                """
            from __future__ import annotations

            from dataclasses import dataclass, field
            from typing import TYPE_CHECKING

            if TYPE_CHECKING:
                from .auth import AuthProvider


            @dataclass(frozen=True)
            class ClientConfig:
                host: str = "localhost:50051"
                secure: bool = False
                timeout: float | None = 10.0
                metadata: tuple[tuple[str, str], ...] = field(default_factory=tuple)
                auth_provider: "AuthProvider | None" = None
                root_certificates: bytes | None = None
            """
            ).strip()
            + "\n"
        )

    def _render_errors(self) -> str:
        return (
            textwrap.dedent(
                """
            from __future__ import annotations

            import grpc


            class SDKError(Exception):
                pass


            class NotFoundError(SDKError):
                pass


            class UnauthorizedError(SDKError):
                pass


            class InvalidArgumentError(SDKError):
                pass


            def map_rpc_error(exc: grpc.RpcError) -> SDKError:
                code = exc.code()
                details = exc.details() if callable(getattr(exc, "details", None)) else str(exc)
                if code == grpc.StatusCode.NOT_FOUND:
                    return NotFoundError(details)
                if code in {grpc.StatusCode.UNAUTHENTICATED, grpc.StatusCode.PERMISSION_DENIED}:
                    return UnauthorizedError(details)
                if code == grpc.StatusCode.INVALID_ARGUMENT:
                    return InvalidArgumentError(details)
                return SDKError(details)
            """
            ).strip()
            + "\n"
        )

    def _group_services_by_app(
        self, specs: list[ServiceSpec]
    ) -> dict[str, list[ServiceSpec]]:
        grouped: dict[str, list[ServiceSpec]] = {}
        for spec in specs:
            grouped.setdefault(spec.sdk_app, []).append(spec)
        return grouped

    def _group_messages_by_app(
        self, specs: list[MessageSpec]
    ) -> dict[str, list[MessageSpec]]:
        grouped: dict[str, list[MessageSpec]] = {}
        for spec in specs:
            grouped.setdefault(spec.sdk_app, []).append(spec)
        return grouped

    def _render_service_base(self) -> str:
        return (
            textwrap.dedent(
                """
            from __future__ import annotations

            from collections.abc import Iterable
            from typing import Any

            import grpc

            from .errors import map_rpc_error


            class BaseServiceClient:
                def __init__(self, grpc_client) -> None:
                    self._grpc_client = grpc_client
                    self._stub = None

                def _get_stub(self):
                    if self._stub is None:
                        self._stub = self.STUB_CLASS(self._grpc_client.channel)
                    return self._stub

                def _resolve_request_class(self, request_type: str):
                    if request_type == "google.protobuf.Empty":
                        from google.protobuf.empty_pb2 import Empty

                        return Empty
                    return getattr(self.PB2_MODULE, request_type.split(".")[-1], None)

                def _coerce_request(self, request_type: str, request: Any):
                    request_cls = self._resolve_request_class(request_type)
                    if request is None:
                        return request_cls() if request_cls is not None else None
                    if request_cls is not None and isinstance(request, request_cls):
                        return request
                    if request_cls is not None and isinstance(request, dict):
                        return request_cls(**request)
                    return request

                def _coerce_stream_request(self, request_type: str, request_iter: Iterable[Any]):
                    for item in request_iter:
                        yield self._coerce_request(request_type, item)

                def _invoke(self, method_name: str, request_type: str, request: Any, *, timeout=None, metadata=None):
                    stub = self._get_stub()
                    method = getattr(stub, method_name)
                    merged_metadata = self._grpc_client._build_metadata(metadata)
                    final_timeout = self._grpc_client.config.timeout if timeout is None else timeout
                    payload = self._coerce_request(request_type, request)
                    try:
                        return method(payload, timeout=final_timeout, metadata=merged_metadata)
                    except grpc.RpcError as exc:
                        raise map_rpc_error(exc) from exc

                def _invoke_stream(self, method_name: str, request_type: str, request_iter: Iterable[Any], *, timeout=None, metadata=None):
                    stub = self._get_stub()
                    method = getattr(stub, method_name)
                    merged_metadata = self._grpc_client._build_metadata(metadata)
                    final_timeout = self._grpc_client.config.timeout if timeout is None else timeout
                    payload = self._coerce_stream_request(request_type, request_iter)
                    try:
                        return method(payload, timeout=final_timeout, metadata=merged_metadata)
                    except grpc.RpcError as exc:
                        raise map_rpc_error(exc) from exc
            """
            ).strip()
            + "\n"
        )

    def _render_app_services(self, specs: list[ServiceSpec]) -> str:
        import_lines: list[str] = [
            "from __future__ import annotations",
            "",
            "from typing import Any",
            "",
            "from ..._service_base import BaseServiceClient",
        ]
        seen_stub_imports: set[tuple[str, str, str]] = set()
        seen_pb2_imports: set[tuple[str, str]] = set()
        for spec in specs:
            stub_import = (spec.stub_module, spec.stub_class, spec.stub_alias)
            if stub_import not in seen_stub_imports:
                import_lines.append(
                    f"from {spec.stub_module} import {spec.stub_class} as {spec.stub_alias}"
                )
                seen_stub_imports.add(stub_import)
            pb2_import = (spec.pb2_module, spec.pb2_alias)
            if pb2_import not in seen_pb2_imports:
                import_lines.append(f"import {spec.pb2_module} as {spec.pb2_alias}")
                seen_pb2_imports.add(pb2_import)
        lines = [*import_lines, "", ""]
        exports: list[str] = []
        for spec in specs:
            if exports:
                lines.extend(["", ""])
            lines.extend(
                [
                    f"class {spec.class_name}(BaseServiceClient):",
                    f"    STUB_CLASS = {spec.stub_alias}",
                    f"    PB2_MODULE = {spec.pb2_alias}",
                ]
            )
            for method in spec.methods:
                request_ref = self._message_ref(method.request_type, spec.pb2_alias)
                response_ref = self._message_ref(method.response_type, spec.pb2_alias)
                if method.client_streaming:
                    return_ref = (
                        f"Iterable[{response_ref}]"
                        if method.server_streaming
                        else response_ref
                    )
                    lines.extend(
                        [
                            "",
                            f"    def {method.python_name}(self, request: Iterable[{request_ref} | dict[str, Any]], *, timeout=None, metadata=None) -> {return_ref}:",
                            f'        return self._invoke_stream("{method.name}", "{method.request_type}", request, timeout=timeout, metadata=metadata)',
                        ]
                    )
                else:
                    return_ref = (
                        f"Iterable[{response_ref}]"
                        if method.server_streaming
                        else response_ref
                    )
                    lines.extend(
                        [
                            "",
                            f"    def {method.python_name}(self, request: {request_ref} | dict[str, Any] | None = None, *, timeout=None, metadata=None) -> {return_ref}:",
                            f'        return self._invoke("{method.name}", "{method.request_type}", request, timeout=timeout, metadata=metadata)',
                        ]
                    )
            exports.append(spec.class_name)
        lines.extend(["", "", f"__all__ = {exports!r}", ""])
        return "\n".join(lines)

    def _render_services_index(self, by_app: dict[str, list[ServiceSpec]]) -> str:
        lines = [
            "from __future__ import annotations",
            "",
            "from ._service_base import BaseServiceClient",
        ]
        for app in sorted(by_app):
            classes = ", ".join(spec.class_name for spec in by_app[app])
            if classes:
                lines.append(f"from .generated.{app}.services import {classes}")
        lines.extend(["", "SERVICE_CLIENTS = {"])
        for app in sorted(by_app):
            for spec in by_app[app]:
                lines.append(f'    "{spec.python_attr}": {spec.class_name},')
        lines.extend(["}", "", ""])
        exports = [spec.class_name for app in sorted(by_app) for spec in by_app[app]]
        lines.append(
            f"__all__ = {[*exports, 'BaseServiceClient', 'SERVICE_CLIENTS']!r}"
        )
        lines.append("")
        return "\n".join(lines)

    def _render_app_models(self, specs: list[MessageSpec]) -> str:
        lines = [
            "from __future__ import annotations",
            "",
            "from datetime import datetime",
            "",
            "from pydantic import BaseModel, ConfigDict, Field",
            "",
            "",
        ]
        exports: list[str] = []
        for spec in specs:
            if exports:
                lines.extend(["", ""])
            lines.extend(
                [
                    f"class {spec.model_name}(BaseModel):",
                    "    model_config = ConfigDict(extra='ignore', populate_by_name=True)",
                ]
            )
            if not spec.fields:
                lines.append("    pass")
            else:
                for field in spec.fields:
                    lines.append(
                        f"    {field.name}: {field.annotation} = {field.default_expr}"
                    )
            exports.append(spec.model_name)
        lines.extend(["", "", f"__all__ = {exports!r}", ""])
        return "\n".join(lines)

    def _render_models_index(self, by_app: dict[str, list[MessageSpec]]) -> str:
        lines = ["from __future__ import annotations", ""]
        exports: list[str] = []
        for app in sorted(by_app):
            classes = ", ".join(spec.model_name for spec in by_app[app])
            if classes:
                lines.append(f"from .generated.{app}.models import {classes}")
                exports.extend(spec.model_name for spec in by_app[app])
        lines.extend(["", "", f"__all__ = {exports!r}", ""])
        return "\n".join(lines)

    def _render_typed_base(self) -> str:
        return (
            textwrap.dedent(
                """
            from __future__ import annotations

            from collections.abc import Iterable
            from typing import Any

            from .helpers import message_to_dict


            class BaseTypedServiceClient:
                def __init__(self, raw_service) -> None:
                    self._raw = raw_service

                def _to_payload(self, request: Any):
                    if request is None:
                        return None
                    if hasattr(request, "model_dump"):
                        return request.model_dump(exclude_none=True, by_alias=True)
                    return request

                def _to_model(self, model_cls, response: Any):
                    if model_cls is Any:
                        return response
                    return model_cls.model_validate(message_to_dict(response))

                def _to_model_stream(self, model_cls, response_stream: Iterable[Any]):
                    for item in response_stream:
                        yield self._to_model(model_cls, item)
            """
            ).strip()
            + "\n"
        )

    def _render_app_typed_services(
        self,
        specs: list[ServiceSpec],
        model_name_map: dict[tuple[str, str], str],
    ) -> str:
        lines = [
            "from __future__ import annotations",
            "",
            "from collections.abc import Iterable",
            "from typing import Any",
            "",
            "from ... import models",
            "from ..._typed_base import BaseTypedServiceClient",
        ]
        raw_classes = ", ".join(spec.class_name for spec in specs)
        if raw_classes:
            lines.append(f"from .services import {raw_classes}")
        lines.extend(["", ""])
        exports: list[str] = []
        for spec in specs:
            typed_class_name = f"Typed{spec.class_name}"
            if exports:
                lines.extend(["", ""])
            lines.extend(
                [
                    f"class {typed_class_name}(BaseTypedServiceClient):",
                    "    def __init__(self, grpc_client) -> None:",
                    f"        super().__init__({spec.class_name}(grpc_client))",
                ]
            )
            for method in spec.methods:
                req_key = (spec.pb2_alias, method.request_type.split(".")[-1])
                res_key = (spec.pb2_alias, method.response_type.split(".")[-1])
                req_model_name = (
                    model_name_map[req_key]
                    if method.request_type != "google.protobuf.Empty"
                    and req_key in model_name_map
                    else "Any"
                )
                if method.response_type == "google.protobuf.Empty":
                    res_model_name = "Any"
                else:
                    res_model_name = model_name_map.get(res_key, "Any")
                request_type_hint = (
                    "None"
                    if method.request_type == "google.protobuf.Empty"
                    else f"models.{req_model_name} | dict[str, Any] | None"
                )
                response_type_hint = (
                    f"Iterable[models.{res_model_name}]"
                    if method.server_streaming and res_model_name != "Any"
                    else (
                        "Iterable[Any]"
                        if method.server_streaming
                        else f"models.{res_model_name}"
                    )
                    if res_model_name != "Any"
                    else ("Iterable[Any]" if method.server_streaming else "Any")
                )
                lines.extend(
                    [
                        "",
                        f"    def {method.python_name}(self, request: {request_type_hint} = None, *, timeout=None, metadata=None) -> {response_type_hint}:",
                        "        payload = self._to_payload(request)",
                        f"        response = self._raw.{method.python_name}(payload, timeout=timeout, metadata=metadata)",
                        f"        model_cls = models.{res_model_name} if '{res_model_name}' != 'Any' else Any",
                        "        if isinstance(response, Iterable) and not isinstance(response, (bytes, str, dict)):",
                        "            return self._to_model_stream(model_cls, response)",
                        "        return self._to_model(model_cls, response)",
                    ]
                )
            exports.append(typed_class_name)
        lines.extend(["", "", f"__all__ = {exports!r}", ""])
        return "\n".join(lines)

    def _render_typed_services_index(self, by_app: dict[str, list[ServiceSpec]]) -> str:
        lines = ["from __future__ import annotations", "", "from typing import Any", ""]
        for app in sorted(by_app):
            classes = ", ".join(f"Typed{spec.class_name}" for spec in by_app[app])
            if classes:
                lines.append(f"from .generated.{app}.typed_services import {classes}")
        lines.extend(
            [
                "",
                "",
                "class TypedGrpcClient:",
                "    def __init__(self, grpc_client) -> None:",
                "        self._grpc_client = grpc_client",
                "        self._cache: dict[str, Any] = {}",
                "",
                "    def service(self, name: str):",
                "        if name not in self._cache:",
                "            cls = TYPED_SERVICE_CLIENTS[name]",
                "            self._cache[name] = cls(self._grpc_client)",
                "        return self._cache[name]",
            ]
        )
        for app in sorted(by_app):
            for spec in by_app[app]:
                typed_name = f"Typed{spec.class_name}"
                lines.extend(
                    [
                        "",
                        "    @property",
                        f"    def {spec.python_attr}(self) -> {typed_name}:",
                        f'        return self.service("{spec.python_attr}")',
                    ]
                )
        lines.extend(["", "", "TYPED_SERVICE_CLIENTS = {"])
        for app in sorted(by_app):
            for spec in by_app[app]:
                lines.append(f'    "{spec.python_attr}": Typed{spec.class_name},')
        lines.extend(["}", "", "__all__ = ['TypedGrpcClient']", ""])
        return "\n".join(lines)

    def _message_ref(self, message_type: str, pb2_alias: str) -> str:
        if message_type == "google.protobuf.Empty":
            return "Any"
        return f"{pb2_alias}.{message_type.split('.')[-1]}"

    def _render_generated_client(self, specs: list[ServiceSpec]) -> str:
        service_imports = ", ".join(spec.class_name for spec in specs)
        services_import_line = (
            f"from .services import SERVICE_CLIENTS, {service_imports}"
            if service_imports
            else "from .services import SERVICE_CLIENTS"
        )
        lines = [
            "from __future__ import annotations",
            "",
            "from typing import Any, cast",
            "",
            "import grpc",
            "",
            "from .config import ClientConfig",
            services_import_line,
            "from .typed_services import TypedGrpcClient",
            "",
            "",
            "class GeneratedGrpcClient:",
            "    def __init__(self, config: ClientConfig | None = None) -> None:",
            "        self.config = config or ClientConfig()",
            "        self._channel = None",
            "        self._service_cache: dict[str, Any] = {}",
            "        self._typed = None",
            "",
            "    @property",
            "    def channel(self):",
            "        if self._channel is None:",
            "            if self.config.secure:",
            "                creds = grpc.ssl_channel_credentials(",
            "                    root_certificates=self.config.root_certificates",
            "                )",
            "                self._channel = grpc.secure_channel(self.config.host, creds)",
            "            else:",
            "                self._channel = grpc.insecure_channel(self.config.host)",
            "        return self._channel",
            "",
            "    def _build_metadata(self, metadata):",
            "        base = list(self.config.metadata)",
            "        auth_provider = self.config.auth_provider",
            "        if auth_provider is not None:",
            "            base.extend(auth_provider.get_metadata())",
            "        if metadata:",
            "            base.extend(list(metadata))",
            "        return tuple(base)",
            "",
            "    def service(self, name: str):",
            "        if name not in self._service_cache:",
            "            service_cls = SERVICE_CLIENTS[name]",
            "            self._service_cache[name] = service_cls(self)",
            "        return self._service_cache[name]",
            "",
            "    @property",
            "    def typed(self) -> TypedGrpcClient:",
            "        if self._typed is None:",
            "            self._typed = TypedGrpcClient(self)",
            "        return self._typed",
        ]
        for spec in specs:
            lines.extend(
                [
                    "",
                    "    @property",
                    f"    def {spec.python_attr}(self) -> {spec.class_name}:",
                    f'        return cast({spec.class_name}, self.service("{spec.python_attr}"))',
                ]
            )
        lines.extend(
            [
                "",
                "    def close(self) -> None:",
                "        if self._channel is not None:",
                "            self._channel.close()",
                "            self._channel = None",
                "",
                "    def __getattr__(self, item: str):",
                "        if item in SERVICE_CLIENTS:",
                "            return self.service(item)",
                "        raise AttributeError(item)",
                "",
            ]
        )
        return "\n".join(lines)

    def _render_client_wrapper(self) -> str:
        return (
            textwrap.dedent(
                """
            from __future__ import annotations

            from .client_generated import GeneratedGrpcClient


            class GrpcClient(GeneratedGrpcClient):
                \"\"\"Customizable SDK client.

                Keep project-specific logic here; generator will not overwrite this file
                when target SDK directory already exists.
                \"\"\"
            """
            ).strip()
            + "\n"
        )

    def _render_helpers(self) -> str:
        return (
            textwrap.dedent(
                """
            from __future__ import annotations

            import inspect
            from typing import Any

            from google.protobuf.json_format import MessageToDict


            def message_to_dict(message: Any) -> dict[str, Any]:
                \"\"\"Convert protobuf message to plain Python dict.\"\"\"
                kwargs: dict[str, Any] = {"preserving_proto_field_name": True}
                params = inspect.signature(MessageToDict).parameters
                if "always_print_fields_with_no_presence" in params:
                    kwargs["always_print_fields_with_no_presence"] = True
                elif "including_default_value_fields" in params:
                    kwargs["including_default_value_fields"] = True
                return MessageToDict(message, **kwargs)


            def extract_results(message_or_dict: Any) -> list[Any]:
                \"\"\"Return paginated `results` list when present, otherwise empty list.\"\"\"
                if isinstance(message_or_dict, dict):
                    payload = message_or_dict
                else:
                    payload = message_to_dict(message_or_dict)
                value = payload.get("results", [])
                return value if isinstance(value, list) else []
            """
            ).strip()
            + "\n"
        )

    def _render_pyproject(self, sdk_name: str, package_name: str) -> str:
        return (
            textwrap.dedent(
                f"""
            [build-system]
            requires = ["setuptools>=68", "wheel"]
            build-backend = "setuptools.build_meta"

            [project]
            name = "{sdk_name}"
            version = "0.1.0"
            description = "Generated gRPC Python SDK"
            readme = "README.md"
            requires-python = ">=3.10"
            dependencies = [
              "grpcio>=1.50",
              "protobuf>=4.25",
              "pydantic>=2.0",
            ]

            [tool.setuptools]
            package-dir = {{"" = "src"}}
            include-package-data = true

            [tool.setuptools.packages.find]
            where = ["src"]
            include = ["*"]
            """
            ).strip()
            + "\n"
        )

    def _render_readme(self, sdk_name: str, package_name: str) -> str:
        return (
            textwrap.dedent(
                f"""
            # {sdk_name}

            Generated Python gRPC SDK.

            This SDK has two access layers:
            - raw protobuf client (`client.<service>.<method>()`)
            - typed client (`client.typed.<service>.<method>()`) based on generated Pydantic models

            ## Quickstart

            ```python
            from {package_name} import (
                ClientConfig,
                GrpcClient,
                StaticTokenAuth,
                message_to_dict,
                extract_results,
            )

            client = GrpcClient(
                ClientConfig(
                    host="localhost:50051",
                    auth_provider=StaticTokenAuth("<token>"),
                )
            )

            # Raw protobuf response
            raw_resp = client.product.list({{"limit": 20}})
            raw_payload = message_to_dict(raw_resp)
            rows = extract_results(raw_resp)  # paginated list helper

            # Typed response (Pydantic models generated from proto)
            typed_resp = client.typed.product.list({{"limit": 20}})
            ```

            ## Generated layout

            ```text
            src/{package_name}/
              client.py                  # custom layer (not overwritten if exists)
              client_generated.py        # regenerated
              helpers.py                 # helper functions (not overwritten if exists)
              services.py                # facade, regenerated
              typed_services.py          # facade, regenerated
              models.py                  # facade, regenerated
              generated/
                <app>/
                  services.py            # regenerated
                  typed_services.py      # regenerated
                  models.py              # regenerated
            ```

            ## Notes

            - `client.py` is safe for custom logic.
            - `helpers.py` is safe for custom additions.
            - files under `generated/` are fully regenerated.
            """
            ).strip()
            + "\n"
        )


class PhpClientSDKGenerator(BaseClientSDKGenerator):
    language = "php"

    def generate(
        self,
        *,
        proto_files: Iterable[Path],
        out_dir: Path,
        sdk_name: str,
        include_root: Path,
    ) -> Path:
        protoc_bin = shutil.which("protoc")
        if not protoc_bin:
            raise SDKGenerationError(
                "protoc is required for PHP SDK generation and was not found in PATH."
            )

        configured = getattr(django_settings, "GRPC_EXTRA", {}) or {}
        plugin_path = configured.get("PHP_GRPC_PLUGIN")
        if not plugin_path:
            raise SDKGenerationError(
                "GRPC_EXTRA['PHP_GRPC_PLUGIN'] must point to grpc_php_plugin."
            )
        plugin = Path(str(plugin_path))
        if not plugin.exists():
            raise SDKGenerationError(f"grpc_php_plugin was not found: {plugin}")

        target_dir = out_dir / sdk_name
        target_dir.mkdir(parents=True, exist_ok=True)

        for proto_file in proto_files:
            args = [
                protoc_bin,
                f"-I{include_root}",
                f"--php_out={target_dir}",
                f"--grpc_out={target_dir}",
                f"--plugin=protoc-gen-grpc={plugin}",
                str(proto_file),
            ]
            completed = subprocess.run(
                args,
                capture_output=True,
                text=True,
                check=False,
            )
            if completed.returncode != 0:
                raise SDKGenerationError(
                    f"protoc failed for {proto_file}: {completed.stderr.strip()}"
                )

        return target_dir



from __future__ import annotations

import argparse
from collections.abc import Iterable
from contextlib import contextmanager
from typing import Any

from django.conf import settings as django_settings
from django.core.management.base import BaseCommand, CommandError
from django.utils.module_loading import import_string

from grpc_extra.main import GrpcExtra


class Command(BaseCommand):
    help = "Run gRPC server using GrpcExtra runtime."

    def add_arguments(self, parser):
        parser.add_argument("--bind", help="Bind address, e.g. [::]:50051.")
        parser.add_argument("--max-workers", type=int, help="Thread pool size.")
        parser.add_argument("--max-msg-mb", type=int, help="Max message size in MB.")
        parser.add_argument(
            "--health",
            action=argparse.BooleanOptionalAction,
            default=None,
            help="Enable/disable health service.",
        )
        parser.add_argument(
            "--reflection",
            action=argparse.BooleanOptionalAction,
            default=None,
            help="Enable/disable reflection service.",
        )
        parser.add_argument(
            "--reload",
            action=argparse.BooleanOptionalAction,
            default=None,
            help="Enable/disable live reload (requires watchfiles).",
        )
        parser.add_argument(
            "--discover",
            action=argparse.BooleanOptionalAction,
            default=True,
            help="Enable/disable auto-discovery of gRPC services.",
        )
        parser.add_argument(
            "--reload-path",
            action="append",
            default=[],
            help="Path to watch in reload mode (repeatable).",
        )
        parser.add_argument(
            "--auth-backend",
            help="Auth backend import path, e.g. path.to.auth_backend.",
        )
        parser.add_argument(
            "--interceptor",
            action="append",
            default=[],
            help="Interceptor import path (instance or zero-arg class). Repeatable.",
        )
        parser.add_argument(
            "--request-logging",
            action=argparse.BooleanOptionalAction,
            default=None,
            help="Enable/disable request logging interceptor.",
        )
        parser.add_argument(
            "--logger-name",
            help="Logger name for request logging (default: grpc_extra).",
        )

    def handle(self, *args, **options):
        interceptors = self._resolve_interceptors(options.get("interceptor", []))
        runtime_kwargs = {
            "bind": options.get("bind"),
            "max_workers": options.get("max_workers"),
            "max_msg_mb": options.get("max_msg_mb"),
            "enable_health": options.get("health"),
            "enable_reflection": options.get("reflection"),
            "auth_backend": options.get("auth_backend"),
            "interceptors": interceptors if interceptors else None,
            "reload": options.get("reload"),
            "reload_paths": options.get("reload_path") or None,
            "auto_discover": options.get("discover", True),
        }
        runtime_kwargs = {
            key: value for key, value in runtime_kwargs.items() if value is not None
        }

        settings_overrides = {}
        if options.get("request_logging") is not None:
            settings_overrides["ENABLE_REQUEST_LOGGING"] = options["request_logging"]
        if options.get("logger_name"):
            settings_overrides["LOGGER_NAME"] = options["logger_name"]

        with self._override_grpc_settings(**settings_overrides):
            GrpcExtra().run_server(**runtime_kwargs)

    def _resolve_interceptors(self, paths: Iterable[str]) -> list[object]:
        resolved: list[object] = []
        for path in paths:
            try:
                value = import_string(path)
            except Exception as exc:
                raise CommandError(f"Invalid interceptor path '{path}': {exc}") from exc
            if isinstance(value, type):
                try:
                    value = value()
                except Exception as exc:
                    raise CommandError(
                        f"Interceptor class '{path}' must be instantiable without args."
                    ) from exc
            resolved.append(value)
        return resolved

    @contextmanager
    def _override_grpc_settings(self, **patch: Any):
        current = getattr(django_settings, "GRPC_EXTRA", {}) or {}
        merged = dict(current)
        merged.update(patch)
        django_settings.GRPC_EXTRA = merged
        try:
            yield
        finally:
            django_settings.GRPC_EXTRA = current



from __future__ import annotations

import inspect
from collections.abc import Callable, Iterable
from typing import Any, Type, get_args, get_origin

from pydantic import BaseModel, create_model

from .constants import (
    GRPC_METHOD_META,
    GRPC_ORDERING_META,
    GRPC_PAGINATION_META,
    GRPC_PERMISSIONS_META,
    GRPC_SEARCHING_META,
    GRPC_SERVICE_META,
)
from .ordering import BaseOrdering, get_default_ordering_class, resolve_ordering_class
from .permissions import PermissionLike, resolve_permissions
from .pagination import (
    BasePagination,
    get_default_pagination_class,
    resolve_pagination_class,
)
from .searching import (
    BaseSearching,
    get_default_searching_class,
    resolve_searching_class,
)
from .registry import MethodMeta, ServiceMeta, registry
from .utils import to_upper_camel_case


def grpc_service(
    *,
    name: str | None = None,
    app_label: str | None = None,
    package: str | None = None,
    proto_path: str | None = None,
    description: str | None = None,
    factory: Callable[[], object] | None = None,
    permissions: Iterable[PermissionLike] | None = None,
) -> Callable[[Type], Type]:
    """Declare a class as a gRPC service and register its metadata.

    The decorator stores service metadata on the class and registers it in the
    global registry so it can be discovered by proto generation and server setup.

    Args:
        name: Public gRPC service name. Defaults to class name.
        app_label: Django app label used to resolve proto module paths.
        package: Proto package name. Defaults to app label.
        proto_path: Path to the proto file relative to app root.
        description: Optional service description for documentation.
        factory: Optional callable used to create the servicer instance.
    """

    def decorator(service_cls: Type) -> Type:
        service_name = name or service_cls.__name__
        resolved_app_label = app_label or service_cls.__module__.split(".")[0]
        resolved_package = package or resolved_app_label
        resolved_proto_path = proto_path or f"grpc/proto/{resolved_app_label}.proto"
        resolved_description = description or inspect.getdoc(service_cls)
        meta = ServiceMeta(
            name=service_name,
            app_label=resolved_app_label,
            package=resolved_package,
            proto_path=resolved_proto_path,
            description=resolved_description,
            factory=factory,
            permissions=resolve_permissions(permissions),
        )
        setattr(service_cls, GRPC_SERVICE_META, meta)
        registry.register(service_cls)
        return service_cls

    return decorator


def grpc_method(
    *,
    name: str | None = None,
    request_schema: Type[BaseModel] | None = None,
    response_schema: Type[BaseModel] | None = None,
    description: str | None = None,
    client_streaming: bool = False,
    server_streaming: bool = False,
    permissions: Iterable[PermissionLike] | None = None,
) -> Callable[[Callable], Callable]:
    """Declare a method as an RPC endpoint and attach generation metadata.

    Args:
        name: RPC method name. Defaults to function name converted to
            UpperCamelCase.
        request_schema: Pydantic model for request message generation.
        response_schema: Pydantic model for response message generation.
        description: Optional method description for documentation.
        client_streaming: Whether request is a stream.
        server_streaming: Whether response is a stream.
    """

    def decorator(method: Callable) -> Callable:
        method_name = name or to_upper_camel_case(method.__name__)
        resolved_description = description or inspect.getdoc(method)
        searching_meta = getattr(method, GRPC_SEARCHING_META, None)
        ordering_meta = getattr(method, GRPC_ORDERING_META, None)
        pagination_class = getattr(method, GRPC_PAGINATION_META, None)
        has_method_permission_meta = hasattr(method, GRPC_PERMISSIONS_META)
        permissions_meta = getattr(method, GRPC_PERMISSIONS_META, ())
        permissions_overridden = has_method_permission_meta or permissions is not None
        request_schema_resolved = _resolve_top_level_collection_schema(
            request_schema, direction="request"
        )
        response_schema_resolved = _resolve_top_level_collection_schema(
            response_schema, direction="response"
        )
        searching_handler = None
        ordering_handler = None

        if searching_meta is not None:
            searching_class, searching_params = searching_meta
            searching_handler = searching_class(**searching_params)
            request_schema_resolved = searching_handler.build_request_schema(
                request_schema_resolved
            )
        if ordering_meta is not None:
            ordering_class, ordering_params = ordering_meta
            ordering_handler = ordering_class(**ordering_params)
            request_schema_resolved = ordering_handler.build_request_schema(
                request_schema_resolved
            )

        if pagination_class is not None:
            if client_streaming or server_streaming:
                raise ValueError(
                    "grpc_pagination can be used only with unary endpoints."
                )
            if response_schema_resolved is None:
                raise ValueError(
                    "grpc_pagination requires `response_schema` in grpc_method."
                )
            request_schema_resolved = pagination_class.build_request_schema(
                request_schema_resolved
            )
            response_schema_resolved = pagination_class.build_response_schema(
                response_schema_resolved
            )
        meta = MethodMeta(
            name=method_name,
            handler_name=method.__name__,
            request_schema=request_schema_resolved,
            response_schema=response_schema_resolved,
            pagination_class=pagination_class,
            ordering_handler=ordering_handler,
            searching_handler=searching_handler,
            description=resolved_description,
            client_streaming=client_streaming,
            server_streaming=server_streaming,
            permissions=(
                resolve_permissions(permissions_meta) + resolve_permissions(permissions)
            ),
            permissions_overridden=permissions_overridden,
        )
        setattr(method, GRPC_METHOD_META, meta)
        return method

    return decorator


def _resolve_top_level_collection_schema(
    schema: Type[BaseModel] | None | Any,
    *,
    direction: str,
) -> Type[BaseModel] | None:
    if schema is None:
        return None
    origin = get_origin(schema)
    args = get_args(schema)
    if origin is not list:
        return schema
    if len(args) != 1:
        raise ValueError("List[T] schema must define exactly one item type.")
    item_schema = args[0]
    if not isinstance(item_schema, type) or not issubclass(item_schema, BaseModel):
        raise ValueError("List[T] schema requires T to be a Pydantic model.")
    wrapper_name = f"{item_schema.__name__}List{direction.title()}Schema"
    item_list_type = list[item_schema]  # type: ignore[valid-type]
    return create_model(wrapper_name, items=(item_list_type, ...))


def grpc_pagination(
    pagination_class: Callable | type[BasePagination] | str | None = None,
) -> Callable[[Callable], Callable] | Callable:
    """Attach pagination behavior to a grpc method.

    This decorator must be placed under `@grpc_method`.
    """

    def decorator(method: Callable) -> Callable:
        if getattr(method, GRPC_METHOD_META, None) is not None:
            raise ValueError("grpc_pagination must be placed under @grpc_method.")
        resolved = (
            get_default_pagination_class()
            if pagination_class is None
            else resolve_pagination_class(pagination_class)
        )
        setattr(method, GRPC_PAGINATION_META, resolved)
        return method

    if callable(pagination_class) and not inspect.isclass(pagination_class):
        method = pagination_class
        pagination_class = None
        return decorator(method)
    return decorator


def grpc_ordering(
    ordering_class: Callable | type[BaseOrdering] | str | None = None,
    **ordering_params: Any,
) -> Callable[[Callable], Callable] | Callable:
    """Attach ordering behavior to a grpc method."""

    def decorator(method: Callable) -> Callable:
        if getattr(method, GRPC_METHOD_META, None) is not None:
            raise ValueError("grpc_ordering must be placed under @grpc_method.")
        resolved = (
            get_default_ordering_class()
            if ordering_class is None
            else resolve_ordering_class(ordering_class)
        )
        if resolved is None:
            raise ValueError("Ordering class cannot be None.")
        params = _resolve_modifier_fields_params(
            ordering_params,
            resolved=resolved,
            legacy_fields_key="ordering_fields",
            decorator_name="grpc_ordering",
        )
        setattr(method, GRPC_ORDERING_META, (resolved, params))
        return method

    if callable(ordering_class) and not inspect.isclass(ordering_class):
        method = ordering_class
        ordering_class = None
        return decorator(method)
    return decorator


def grpc_searching(
    searching_class: Callable | type[BaseSearching] | str | None = None,
    **searching_params: Any,
) -> Callable[[Callable], Callable] | Callable:
    """Attach searching behavior to a grpc method."""

    def decorator(method: Callable) -> Callable:
        if getattr(method, GRPC_METHOD_META, None) is not None:
            raise ValueError("grpc_searching must be placed under @grpc_method.")
        resolved = (
            get_default_searching_class()
            if searching_class is None
            else resolve_searching_class(searching_class)
        )
        if resolved is None:
            raise ValueError("Searching class cannot be None.")
        params = _resolve_modifier_fields_params(
            searching_params,
            resolved=resolved,
            legacy_fields_key="search_fields",
            decorator_name="grpc_searching",
        )
        setattr(method, GRPC_SEARCHING_META, (resolved, params))
        return method

    if callable(searching_class) and not inspect.isclass(searching_class):
        method = searching_class
        searching_class = None
        return decorator(method)
    return decorator


def _has_explicit_fields(value: Any) -> bool:
    if value is None:
        return False
    if isinstance(value, str):
        return bool(value.strip())
    if isinstance(value, Iterable):
        return any(True for _ in value)
    return False


def _resolve_modifier_fields_params(
    raw_params: dict[str, Any],
    *,
    resolved: type[BaseOrdering] | type[BaseSearching],
    legacy_fields_key: str,
    decorator_name: str,
) -> dict[str, Any]:
    params = dict(raw_params)
    fields_param_name = getattr(resolved, "fields_param_name", legacy_fields_key)
    fields_required = bool(getattr(resolved, "fields_required", True))

    alias_keys = {"fields", legacy_fields_key, fields_param_name}
    present_alias_keys = [key for key in alias_keys if key in params]
    if len(present_alias_keys) > 1:
        raise ValueError(
            f"{decorator_name} received multiple field parameter aliases: "
            f"{', '.join(sorted(present_alias_keys))}. "
            "Use only one of them."
        )

    fields_value = None
    if "fields" in params:
        fields_value = params.pop("fields")
    elif legacy_fields_key in params and legacy_fields_key != fields_param_name:
        fields_value = params.pop(legacy_fields_key)
    elif fields_param_name in params:
        fields_value = params[fields_param_name]
    elif legacy_fields_key in params:
        fields_value = params[legacy_fields_key]

    if fields_required and not _has_explicit_fields(fields_value):
        raise ValueError(
            f"{decorator_name} requires fields for {resolved.__name__} "
            f"(expected parameter: '{fields_param_name}')."
        )

    if fields_value is not None and fields_param_name not in params:
        params[fields_param_name] = fields_value

    return params


def grpc_permissions(*permissions: PermissionLike) -> Callable[[Callable], Callable]:
    """Attach permission classes to a grpc method."""

    def decorator(method: Callable) -> Callable:
        if getattr(method, GRPC_METHOD_META, None) is not None:
            raise ValueError("grpc_permissions must be placed under @grpc_method.")
        current = tuple(getattr(method, GRPC_PERMISSIONS_META, ()))
        setattr(method, GRPC_PERMISSIONS_META, current + tuple(permissions))
        return method

    return decorator



from __future__ import annotations

from collections.abc import Iterable, Iterator, Mapping
from dataclasses import asdict, is_dataclass
from decimal import Decimal
from typing import Any, get_args, get_origin, cast

from django.db.models import Model, QuerySet
from pydantic import BaseModel

from .exceptions import RequestDecodeError, ResponseEncodeError


def decode_request_value(value: Any, schema: type[BaseModel] | None) -> Any:
    if schema is None:
        return value
    payload = _to_payload(value)
    try:
        return schema.model_validate(payload)
    except Exception as exc:
        raise RequestDecodeError(f"Failed to decode request: {exc}") from exc


def decode_request_iter(
    request_iter: Iterable[Any], schema: type[BaseModel] | None
) -> Iterator[Any]:
    for item in request_iter:
        yield decode_request_value(item, schema)


def encode_response_value(
    value: Any,
    schema: type[BaseModel] | None,
    response_pb2_cls: type | None,
) -> Any:
    if response_pb2_cls is None:
        return value
    payload = _normalize_response(value, schema)
    try:
        return response_pb2_cls(**payload)
    except Exception as exc:
        raise ResponseEncodeError(f"Failed to encode response: {exc}") from exc


def encode_response_iter(
    values: Iterable[Any],
    schema: type[BaseModel] | None,
    response_pb2_cls: type | None,
) -> Iterator[Any]:
    iterator = getattr(values, "iterator", None)
    if isinstance(values, QuerySet) or callable(iterator):
        iterable: Iterable[Any] = cast(Any, values).iterator()
    else:
        iterable = values
    for item in iterable:
        yield encode_response_value(item, schema, response_pb2_cls)


def _normalize_response(value: Any, schema: type[BaseModel] | None) -> dict[str, Any]:
    if schema is None:
        payload = _to_payload(value)
        if not isinstance(payload, Mapping):
            raise ResponseEncodeError("Response must be mapping-compatible.")
        return cast(dict[str, Any], _coerce_protobuf_compatible(dict(payload)))

    if _is_repeated_items_wrapper_schema(schema) and _is_collection_payload(value):
        validated = schema.model_validate({"items": _materialize_collection(value)})
        return cast(
            dict[str, Any],
            _coerce_protobuf_compatible(
                validated.model_dump(mode="python", by_alias=True, exclude_none=True)
            ),
        )

    if isinstance(value, schema):
        return cast(
            dict[str, Any],
            _coerce_protobuf_compatible(
                value.model_dump(mode="python", by_alias=True, exclude_none=True)
            ),
        )
    if isinstance(value, BaseModel):
        validated = schema.model_validate(value.model_dump(mode="python"))
        return cast(
            dict[str, Any],
            _coerce_protobuf_compatible(
                validated.model_dump(mode="python", by_alias=True, exclude_none=True)
            ),
        )
    if isinstance(value, Mapping):
        validated = schema.model_validate(dict(value))
        return cast(
            dict[str, Any],
            _coerce_protobuf_compatible(
                validated.model_dump(mode="python", by_alias=True, exclude_none=True)
            ),
        )
    if is_dataclass(value) and not isinstance(value, type):
        validated = schema.model_validate(asdict(value))
        return cast(
            dict[str, Any],
            _coerce_protobuf_compatible(
                validated.model_dump(mode="python", by_alias=True, exclude_none=True)
            ),
        )
    if isinstance(value, Model):
        validated = schema.model_validate(value, from_attributes=True)
        return cast(
            dict[str, Any],
            _coerce_protobuf_compatible(
                validated.model_dump(mode="python", by_alias=True, exclude_none=True)
            ),
        )

    validated = schema.model_validate(value, from_attributes=True)
    return cast(
        dict[str, Any],
        _coerce_protobuf_compatible(
            validated.model_dump(mode="python", by_alias=True, exclude_none=True)
        ),
    )


def _to_payload(value: Any) -> Any:
    if isinstance(value, BaseModel):
        return value.model_dump(mode="python", by_alias=True, exclude_none=True)
    if isinstance(value, Mapping):
        return dict(value)
    if is_dataclass(value) and not isinstance(value, type):
        return asdict(value)
    if hasattr(value, "DESCRIPTOR"):
        from google.protobuf.json_format import MessageToDict

        return MessageToDict(
            value,
            preserving_proto_field_name=True,
        )
    return value


def _coerce_protobuf_compatible(value: Any) -> Any:
    if isinstance(value, Decimal):
        return str(value)
    if isinstance(value, Mapping):
        return {k: _coerce_protobuf_compatible(v) for k, v in value.items()}
    if isinstance(value, list):
        return [_coerce_protobuf_compatible(v) for v in value]
    if isinstance(value, tuple):
        return tuple(_coerce_protobuf_compatible(v) for v in value)
    return value


def _is_repeated_items_wrapper_schema(schema: type[BaseModel]) -> bool:
    fields = schema.model_fields
    if tuple(fields.keys()) != ("items",):
        return False
    annotation = fields["items"].annotation
    return get_origin(annotation) is list and len(get_args(annotation)) == 1


def _is_collection_payload(value: Any) -> bool:
    if isinstance(value, (str, bytes, bytearray, Mapping, BaseModel)):
        return False
    iterator = getattr(value, "iterator", None)
    if isinstance(value, QuerySet) or callable(iterator):
        return True
    return isinstance(value, Iterable)


def _materialize_collection(value: Any) -> list[Any]:
    iterator = getattr(value, "iterator", None)
    if isinstance(value, QuerySet) or callable(iterator):
        return list(cast(Any, value).iterator())
    return list(cast(Iterable[Any], value))



from __future__ import annotations

from dataclasses import dataclass, field
from typing import TYPE_CHECKING, Callable, Iterable

from .constants import GRPC_METHOD_META, GRPC_SERVICE_META

if TYPE_CHECKING:
    from pydantic import BaseModel

    from .ordering import BaseOrdering
    from .permissions import BasePermission
    from .pagination import BasePagination
    from .searching import BaseSearching


class RegistryError(Exception):
    pass


class ServiceNotDecoratedError(RegistryError):
    pass


@dataclass(frozen=True)
class ServiceMeta:
    name: str
    app_label: str
    package: str | None = None
    proto_path: str | None = None
    description: str | None = None
    factory: Callable[[], object] | None = None
    permissions: tuple["BasePermission", ...] = ()


@dataclass(frozen=True)
class MethodMeta:
    name: str
    handler_name: str
    request_schema: type["BaseModel"] | None = None
    response_schema: type["BaseModel"] | None = None
    pagination_class: type["BasePagination"] | None = None
    ordering_handler: "BaseOrdering | None" = None
    searching_handler: "BaseSearching | None" = None
    description: str | None = None
    client_streaming: bool = False
    server_streaming: bool = False
    permissions: tuple["BasePermission", ...] = ()
    permissions_overridden: bool = False


@dataclass
class ServiceDefinition:
    service: type
    meta: ServiceMeta
    methods: list[MethodMeta] = field(default_factory=list)


class ServiceRegistry:
    def __init__(self) -> None:
        self._services: list[ServiceDefinition] = []

    def register(self, service: type) -> ServiceDefinition:
        meta: ServiceMeta | None = getattr(service, GRPC_SERVICE_META, None)
        if meta is None:
            raise ServiceNotDecoratedError(
                f"Service '{service.__name__}' must be decorated with @grpc_service."
            )

        existing = self._find(service)
        if existing is not None:
            return existing

        methods = self._collect_methods(service)
        definition = ServiceDefinition(service=service, meta=meta, methods=methods)
        self._services.append(definition)
        return definition

    def all(self) -> Iterable[ServiceDefinition]:
        return list(self._services)

    def clear(self) -> None:
        self._services.clear()

    def _find(self, service: type) -> ServiceDefinition | None:
        for definition in self._services:
            if definition.service is service:
                return definition
        return None

    def _collect_methods(self, service: type) -> list[MethodMeta]:
        collected: list[MethodMeta] = []
        for attr_name in dir(service):
            attr = getattr(service, attr_name)
            meta = getattr(attr, GRPC_METHOD_META, None)
            if meta is not None:
                collected.append(meta)
        return collected


registry = ServiceRegistry()
